/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance;

import java.awt.*;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicSeparatorUI;

import org.jvnet.lafwidget.layout.TransitionLayout;
import org.jvnet.substance.color.ColorScheme;
import org.jvnet.substance.theme.SubstanceTheme;
import org.jvnet.substance.utils.SubstanceCoreUtilities;
import org.jvnet.substance.utils.SubstanceConstants.MenuGutterFillKind;
import org.jvnet.substance.utils.menu.MenuUtilities;

/**
 * UI for separators in <b>Substance</b> look and feel.
 * 
 * @author Kirill Grouchnikov
 */
public class SubstanceSeparatorUI extends BasicSeparatorUI {
	public static ComponentUI createUI(JComponent c) {
		return new SubstanceSeparatorUI();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSeparatorUI#paint(java.awt.Graphics,
	 *      javax.swing.JComponent)
	 */
	@Override
	public void paint(Graphics g, JComponent c) {
		Graphics2D graphics = (Graphics2D) g.create();
		try {
			Component parent = c.getParent();
			if (!(parent instanceof JPopupMenu)) {
				SubstanceTheme theme = SubstanceLookAndFeel.getTheme();
				boolean isDark = SubstanceCoreUtilities.isThemeDark(theme
						.getActiveTheme());
				ColorScheme colorScheme = isDark ? theme.getActiveTheme()
						.getColorScheme() : theme.getDefaultTheme()
						.getColorScheme();

				SubstanceCoreUtilities.paintSeparator(graphics, colorScheme,
						isDark, c.getWidth(), c.getHeight(), ((JSeparator) c)
								.getOrientation());
				//
				// Color foreLight = isDark ? colorScheme.getLightColor()
				// : SubstanceColorUtilities.getInterpolatedColor(
				// colorScheme.getLightColor(), colorScheme
				// .getDarkColor(), 0.8);
				// Color foreDark = isDark ? colorScheme.getExtraLightColor()
				// : SubstanceColorUtilities.getInterpolatedColor(
				// colorScheme.getMidColor(), colorScheme
				// .getDarkColor(), 0.4);
				// Color back = isDark ? colorScheme.getDarkColor() :
				// colorScheme
				// .getUltraLightColor();
				//
				// Dimension s = c.getSize();
				// if (((JSeparator) c).getOrientation() == JSeparator.VERTICAL)
				// {
				// // graphics.setPaint(new GradientPaint(0, 0,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getForeground(), 32), 0, s.height / 2, c
				// // .getForeground()));
				// // graphics.drawLine(0, 0, 0, s.height / 2);
				// // graphics.setPaint(new GradientPaint(0, s.height / 2, c
				// // .getForeground(), 0, s.height,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getForeground(), 32)));
				// // graphics.drawLine(0, s.height / 2, 0, s.height);
				// //
				// // graphics.setPaint(new GradientPaint(1, 0,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getBackground(), 96), 1, s.height / 2,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getBackground(), 196)));
				// // graphics.drawLine(1, 0, 1, s.height / 2);
				// // graphics.setPaint(new GradientPaint(1, s.height / 2,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getBackground(), 196), 1, s.height,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getBackground(), 96)));
				// // graphics.drawLine(1, s.height / 2, 1, s.height);
				// graphics.setPaint(new GradientPaint(0, 0,
				// SubstanceColorUtilities
				// .getAlphaColor(foreLight, 32), 0,
				// s.height / 2, SubstanceColorUtilities
				// .getAlphaColor(foreDark, 240)));
				// graphics.drawLine(0, 0, 0, s.height / 2);
				// graphics.setPaint(new GradientPaint(0, s.height / 2,
				// SubstanceColorUtilities
				// .getAlphaColor(foreDark, 240), 0, s.height,
				// SubstanceColorUtilities
				// .getAlphaColor(foreLight, 32)));
				// graphics.drawLine(0, s.height / 2, 0, s.height);
				//
				// graphics.setPaint(new GradientPaint(1, 0,
				// SubstanceColorUtilities.getAlphaColor(back, 32), 1,
				// s.height / 2, SubstanceColorUtilities
				// .getAlphaColor(back, 196)));
				// graphics.drawLine(1, 0, 1, s.height / 2);
				// graphics.setPaint(new GradientPaint(1, s.height / 2,
				// SubstanceColorUtilities.getAlphaColor(back, 196),
				// 1, s.height, SubstanceColorUtilities.getAlphaColor(
				// back, 32)));
				// graphics.drawLine(1, s.height / 2, 1, s.height);
				// } else // HORIZONTAL
				// {
				// // graphics.setPaint(new GradientPaint(0, 0,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getForeground(), 32), s.width / 2, 0, c
				// // .getForeground()));
				// // graphics.drawLine(0, 0, s.width / 2, 0);
				// // graphics.setPaint(new GradientPaint(s.width / 2, 0, c
				// // .getForeground(), s.width, 0,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getForeground(), 32)));
				// // graphics.drawLine(s.width / 2, 0, s.width, 0);
				// //
				// // graphics.setPaint(new GradientPaint(0, 1,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getBackground(), 32), s.width / 2, 1, c
				// // .getBackground()));
				// // graphics.drawLine(0, 1, s.width / 2, 1);
				// // graphics.setPaint(new GradientPaint(s.width / 2, 1, c
				// // .getBackground(), s.width, 1,
				// // SubstanceColorUtilities.getAlphaColor(c
				// // .getBackground(), 32)));
				// // graphics.drawLine(s.width / 2, 1, s.width, 1);
				// graphics.setPaint(new GradientPaint(0, 0,
				// SubstanceColorUtilities
				// .getAlphaColor(foreLight, 32), s.width / 2,
				// 0, SubstanceColorUtilities.getAlphaColor(foreDark,
				// 240)));
				// graphics.drawLine(0, 0, s.width / 2, 0);
				// graphics.setPaint(new GradientPaint(s.width / 2, 0,
				// SubstanceColorUtilities
				// .getAlphaColor(foreDark, 240), s.width, 0,
				// SubstanceColorUtilities
				// .getAlphaColor(foreLight, 32)));
				// graphics.drawLine(s.width / 2, 0, s.width, 0);
				//
				// graphics.setPaint(new GradientPaint(0, 1,
				// SubstanceColorUtilities.getAlphaColor(back, 32),
				// s.width / 2, 1, SubstanceColorUtilities
				// .getAlphaColor(back, 196)));
				// graphics.drawLine(0, 1, s.width / 2, 1);
				// graphics.setPaint(new GradientPaint(s.width / 2, 1,
				// SubstanceColorUtilities.getAlphaColor(back, 196),
				// s.width, 1, SubstanceColorUtilities.getAlphaColor(
				// back, 32)));
				// graphics.drawLine(s.width / 2, 1, s.width, 1);
				// }
				return;
			}

			int xOffset = MenuUtilities.getTextOffset(graphics, parent);

			Dimension s = c.getSize();
			if (parent.getComponentOrientation().isLeftToRight()) {
				MenuGutterFillKind fillKind = SubstanceCoreUtilities
						.getMenuGutterFillKind();
				if (fillKind != MenuGutterFillKind.NONE) {
					ColorScheme scheme = SubstanceCoreUtilities
							.getActiveScheme(parent);
					Color leftColor = ((fillKind == MenuGutterFillKind.SOFT_FILL) || (fillKind == MenuGutterFillKind.HARD)) ? scheme
							.getUltraLightColor()
							: scheme.getLightColor();
					Color rightColor = ((fillKind == MenuGutterFillKind.SOFT_FILL) || (fillKind == MenuGutterFillKind.SOFT)) ? scheme
							.getUltraLightColor()
							: scheme.getLightColor();
					GradientPaint gp = new GradientPaint(0, 0, leftColor,
							xOffset, 0, rightColor);

					graphics.setComposite(TransitionLayout.getAlphaComposite(
							parent, 0.7f));
					graphics.setPaint(gp);
					graphics.fillRect(0, 0, xOffset - 2, c.getHeight());
				}
				graphics.setComposite(TransitionLayout
						.getAlphaComposite(parent));

				graphics.setColor(c.getForeground());
				graphics.drawLine(xOffset, 0, s.width, 0);

				graphics.setColor(c.getBackground());
				graphics.drawLine(xOffset, 1, s.width, 1);
			} else {
				// fix for defect 125 - support of RTL menus
				MenuGutterFillKind fillKind = SubstanceCoreUtilities
						.getMenuGutterFillKind();
				if (fillKind != MenuGutterFillKind.NONE) {
					ColorScheme scheme = SubstanceCoreUtilities
							.getActiveScheme(parent);
					Color leftColor = ((fillKind == MenuGutterFillKind.HARD_FILL) || (fillKind == MenuGutterFillKind.HARD)) ? scheme
							.getLightColor()
							: scheme.getUltraLightColor();
					Color rightColor = ((fillKind == MenuGutterFillKind.HARD_FILL) || (fillKind == MenuGutterFillKind.SOFT)) ? scheme
							.getLightColor()
							: scheme.getUltraLightColor();
					GradientPaint gp = new GradientPaint(s.width - xOffset, 0,
							leftColor, s.width, 0, rightColor);

					graphics.setComposite(TransitionLayout.getAlphaComposite(
							parent, 0.7f));
					graphics.setPaint(gp);
					graphics.fillRect(s.width - xOffset - 2, 0, s.width, c
							.getHeight());
				}
				graphics.setComposite(TransitionLayout
						.getAlphaComposite(parent));

				graphics.setColor(c.getForeground());
				graphics.drawLine(0, 0, s.width - xOffset - 4, 0);

				graphics.setColor(c.getBackground());
				graphics.drawLine(0, 1, s.width - xOffset - 4, 1);
			}
		} finally {
			graphics.dispose();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSeparatorUI#getPreferredSize(javax.swing.JComponent)
	 */
	@Override
	public Dimension getPreferredSize(JComponent c) {
		if (((JSeparator) c).getOrientation() == SwingConstants.VERTICAL)
			return new Dimension(2, 0);
		else
			return new Dimension(0, 2);
	}

//	/*
//	 * (non-Javadoc)
//	 * 
//	 * @see javax.swing.plaf.basic.BasicSeparatorUI#installDefaults(javax.swing.JSeparator)
//	 */
//	@Override
//	protected void installDefaults(JSeparator s) {
//		super.installDefaults(s);
//
//		SubstanceTheme theme = SubstanceLookAndFeel.getTheme();
//		SubstanceTheme titleTheme = SubstanceLookAndFeel.getTheme()
//				.getActiveTitlePaneTheme();
//		Color foregroundColor = new ColorUIResource(SubstanceColorUtilities
//				.getForegroundColor(theme));
//		Color headerForegroundColor = new ColorUIResource(
//				SubstanceColorUtilities.getForegroundColor(titleTheme));
//		Color backgroundColor = new ColorUIResource(SubstanceColorUtilities
//				.getBackgroundColor(theme));
//		Color headerBackgroundColor = new ColorUIResource(
//				SubstanceColorUtilities.getBackgroundColor(titleTheme));
//
////		Color bg = s.getBackground();
////		if (bg instanceof UIResource) {
////			s.setBackground(new SeparatorColorDelegate(s,
////					headerBackgroundColor, backgroundColor));
////		}
////		Color fg = s.getForeground();
////		if (fg instanceof UIResource) {
////			s.setForeground(new SeparatorColorDelegate(s,
////					headerForegroundColor, foregroundColor));
////		}
//	}
}
