/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicSliderUI;

import org.jvnet.lafwidget.layout.TransitionLayout;
import org.jvnet.lafwidget.utils.FadeStateListener;
import org.jvnet.lafwidget.utils.FadeTracker;
import org.jvnet.lafwidget.utils.FadeTracker.FadeKind;
import org.jvnet.substance.color.ColorScheme;
import org.jvnet.substance.utils.*;
import org.jvnet.substance.utils.icon.SubstanceIconFactory;

/**
 * UI for sliders in <b>Substance</b> look and feel.
 * 
 * @author Kirill Grouchnikov
 */
public class SubstanceSliderUI extends BasicSliderUI implements Trackable {
	/**
	 * Background delegate.
	 */
	private static SubstanceFillBackgroundDelegate bgDelegate = new SubstanceFillBackgroundDelegate();

	/**
	 * Surrogate button model for tracking the thumb transitions.
	 */
	private ButtonModel thumbModel;

	/**
	 * Listener for fade animations.
	 */
	private RolloverControlListener substanceRolloverListener;

	/**
	 * Listener on property change events.
	 */
	private PropertyChangeListener substancePropertyChangeListener;

	/**
	 * Listener for fade animations.
	 */
	protected FadeStateListener substanceFadeStateListener;

	// private MouseMotionListener substanceMouseMotionListener;

	/**
	 * Icon for horizontal sliders.
	 */
	protected Icon horizontalIcon;

	/**
	 * Icon for sliders without labels and ticks.
	 */
	protected Icon roundIcon;

	/**
	 * Icon for vertical sliders.
	 */
	protected Icon verticalIcon;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#createUI(javax.swing.JComponent)
	 */
	public static ComponentUI createUI(JComponent c) {
		return new SubstanceSliderUI((JSlider) c);
	}

	/**
	 * Simple constructor.
	 * 
	 * @param slider
	 *            Slider.
	 */
	public SubstanceSliderUI(JSlider slider) {
		super(null);
		this.thumbModel = new DefaultButtonModel();
		this.thumbModel.setArmed(false);
		this.thumbModel.setSelected(false);
		this.thumbModel.setPressed(false);
		this.thumbModel.setRollover(false);
		this.thumbModel.setEnabled(slider.isEnabled());

		int size = SubstanceSizeUtils.getSliderIconSize();
		this.horizontalIcon = SubstanceIconFactory.getSliderHorizontalIcon(
				size, false);
		this.roundIcon = SubstanceIconFactory.getSliderRoundIcon(size);
		this.verticalIcon = SubstanceIconFactory.getSliderVerticalIcon(size,
				false);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#calculateTrackRect()
	 */
	@Override
	protected void calculateTrackRect() {
		super.calculateTrackRect();
		if (this.slider.getOrientation() == SwingConstants.HORIZONTAL) {
			this.trackRect.y = 3;
		}
	}

	/**
	 * Returns the rectangle of track for painting.
	 * 
	 * @return The rectangle of track for painting.
	 */
	private Rectangle getPaintTrackRect() {
		int trackLeft = 0, trackRight = 0, trackTop = 0, trackBottom = 0;
		if (this.slider.getOrientation() == SwingConstants.HORIZONTAL) {
			trackTop = 7;
			trackBottom = trackTop + this.getTrackWidth() - 1;
			// (this.trackRect.height - 6);// -
			// this.getThumbOverhang();
			// if (this.slider.getPaintTicks())
			// trackBottom++;
			// trackTop = trackBottom - (this.getTrackWidth() - 1);
			trackRight = this.trackRect.width - 1;
			// trackLeft = trackRect.x;
			return new Rectangle(this.trackRect.x + trackLeft, /*
																 * this.trackRect.y +
																 */trackTop, trackRight - trackLeft, trackBottom - trackTop);
		} else {
			if (this.slider.getComponentOrientation().isLeftToRight()) {
				trackLeft = 8;// + (this.trackRect.width -
				// this.getTrackWidth()) / 2;
				// - this.getThumbOverhang();// (this.trackRect.width -
				// this.getThumbOverhang())
				// - this.getTrackWidth();
				trackRight = trackLeft + this.getTrackWidth() - 1;// (this.trackRect.width
				// -
				// this.getThumbOverhang())
				// - 1;
			} else {
				trackRight = this.slider.getWidth() - 8;// (this.trackRect.width
				// +
				// this.getTrackWidth())
				// / 2 - 2;
				// - this.getThumbOverhang();// (this.trackRect.width -
				// this.getThumbOverhang())
				// - this.getTrackWidth();
				trackLeft = trackRight - this.getTrackWidth() + 1;// (this.trackRect.width
				// -
				// this.getThumbOverhang())
				// - 1;
				// trackLeft = this.getThumbOverhang();
				// trackRight = this.getThumbOverhang() + this.getTrackWidth() -
				// 1;
			}
			trackBottom = this.trackRect.height - 1;
			return new Rectangle(trackLeft, this.trackRect.y + trackTop,
					trackRight - trackLeft, trackBottom - trackTop);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#paintTrack(java.awt.Graphics)
	 */
	@Override
	public void paintTrack(Graphics g) {
		Graphics2D graphics = (Graphics2D) g;
		graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		boolean drawInverted = this.drawInverted();

		// Translate to the origin of the painting rectangle
		Rectangle paintRect = this.getPaintTrackRect();
		// System.out.println(paintRect);
		graphics.translate(paintRect.x, paintRect.y);

		// Width and height of the painting rectangle.
		int width = paintRect.width;
		int height = paintRect.height;

		boolean isDark = SubstanceCoreUtilities
				.isThemeDark(SubstanceLookAndFeel.getTheme());
		ColorScheme borderColorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
				.getActiveScheme(this.slider)
				: SubstanceCoreUtilities.getDisabledScheme(this.slider);

		Color borderColor1 = isDark ? borderColorScheme.getMidColor()
				: borderColorScheme.getUltraDarkColor();
		Color borderColor2 = isDark ? borderColorScheme.getMidColor()
				: borderColorScheme.getDarkColor();

		// draw border
		// graphics.setColor(borderColorSchemeEnum.getColorScheme().getDarkColor());
		graphics.setStroke(new BasicStroke((float) 1.1, BasicStroke.CAP_ROUND,
				BasicStroke.JOIN_ROUND));

		int radius = 2;
		if (this.slider.getOrientation() == SwingConstants.HORIZONTAL) {
			graphics.setPaint(new GradientPaint(0, 0, borderColor1, 0,
					height - 1, borderColor2));
			graphics.drawRoundRect(0, 0, width, height, radius, radius);
		} else {
			graphics.setPaint(new GradientPaint(0, 0, borderColor1, width - 1,
					0, borderColor2));
			graphics.drawRoundRect(0, 0, width, height, radius, radius);
		}

		Color fillColor1 = isDark ? borderColorScheme.getUltraLightColor()
				: borderColorScheme.getUltraLightColor();
		Color fillColor2 = isDark ? borderColorScheme.getLightColor()
				: borderColorScheme.getLightColor();

		// fill selected portion
		if (this.slider.isEnabled()) {
			if (this.slider.getOrientation() == SwingConstants.HORIZONTAL) {
				int middleOfThumb = this.thumbRect.x
						+ (this.thumbRect.width / 2) - paintRect.x;
				int fillMinX;
				int fillMaxX;

				graphics.setPaint(new GradientPaint(0, 1, fillColor1, 0,
						height - 1, fillColor2));

				if (drawInverted) {
					fillMinX = middleOfThumb;
					fillMaxX = width - 2;
				} else {
					fillMinX = 1;
					fillMaxX = middleOfThumb;
				}

				graphics.fillRoundRect(fillMinX, 1, fillMaxX - fillMinX + 2,
						height - 1, (radius - 2), (radius - 2));
			} else {
				int middleOfThumb = this.thumbRect.y
						+ (this.thumbRect.height / 2) - paintRect.y;
				int fillMinY;
				int fillMaxY;

				graphics.setPaint(new GradientPaint(1, 0, fillColor1,
						width - 1, 0, fillColor2));

				if (this.drawInverted()) {
					fillMinY = 1;
					fillMaxY = middleOfThumb;
				} else {
					fillMinY = middleOfThumb;
					fillMaxY = height - 2;
				}
				graphics.fillRoundRect(1, fillMinY, width - 1, fillMaxY
						- fillMinY + 2, (radius - 2), (radius - 2));
			}
		}

		g.translate(-paintRect.x, -paintRect.y);
	}

	@Override
	protected Dimension getThumbSize() {
		Icon thumbIcon = this.getIcon();
		// if (slider.getOrientation() == JSlider.HORIZONTAL) {
		// thumbIcon = UIManager.getIcon("Slider.horizontalThumbIcon");
		// } else {
		// thumbIcon = UIManager.getIcon("Slider.verticalThumbIcon");
		// }
		// System.out.println(thumbIcon.getIconWidth() + ":" +
		// thumbIcon.getIconHeight());
		return new Dimension(thumbIcon.getIconWidth(), thumbIcon
				.getIconHeight());
	}

	/**
	 * Returns the thumb icon for the associated slider.
	 * 
	 * @return The thumb icon for the associated slider.
	 */
	protected Icon getIcon() {
		if (this.slider.getOrientation() == JSlider.HORIZONTAL) {
			if (this.slider.getPaintTicks() || this.slider.getPaintLabels())
				return this.horizontalIcon;
			else
				return this.roundIcon;
		} else {
			if (this.slider.getPaintTicks() || this.slider.getPaintLabels())
				return this.verticalIcon;
			else
				return this.roundIcon;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#paintThumb(java.awt.Graphics)
	 */
	public void paintThumb(Graphics g) {
		Graphics2D graphics = (Graphics2D) g.create();
		graphics.setComposite(TransitionLayout.getAlphaComposite(this.slider));
		Rectangle knobBounds = thumbRect;
		// System.out.println(thumbRect);

		graphics.translate(knobBounds.x, knobBounds.y);

		Icon icon = this.getIcon();
		if (slider.getOrientation() == JSlider.HORIZONTAL) {
			int yOffset = 0;
			if (this.slider.getPaintTicks()) {
				yOffset = 2;
			} else {
				yOffset = this.slider.getPaintLabels() ? 1 : -1;
			}
			if (icon != null)
				icon.paintIcon(slider, graphics, -1, yOffset);
		} else {
			if (slider.getComponentOrientation().isLeftToRight()) {
				int xOffset = 0;
				if (this.slider.getPaintTicks())
					xOffset = 4;
				else
					xOffset = this.slider.getPaintLabels() ? 2 : 0;
				if (icon != null)
					icon.paintIcon(slider, graphics, xOffset, -1);
			} else {
				int xOffset = 0;
				if (this.slider.getPaintTicks())
					xOffset = -1;
				else
					xOffset = this.slider.getPaintLabels() ? 2 : 2;
				if (icon != null)
					icon.paintIcon(slider, graphics, xOffset, 1);
			}
		}

		// graphics.translate(-knobBounds.x, -knobBounds.y);
		graphics.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#paint(java.awt.Graphics,
	 *      javax.swing.JComponent)
	 */
	@Override
	public synchronized void paint(Graphics g, JComponent c) {
		// important to synchronize on the slider as we are
		// about to fiddle with its opaqueness
		synchronized (c) {
			SubstanceSliderUI.bgDelegate.update(g, c);
			// remove opaqueness
			boolean isOpaque = c.isOpaque();
			c.setOpaque(false);
			super.paint(g, c);
			// restore opaqueness
			c.setOpaque(isOpaque);
		}

		if (!slider.hasFocus()) {
			if (FadeTracker.getInstance().isTracked(c, FadeKind.FOCUS))
				this.paintFocus(g);
		}

		// g.setColor(Color.green);
		// ((Graphics2D) g).drawRect(0, 0, c.getWidth() - 1, c.getHeight() - 1);
		// g.setColor(Color.blue);
		// ((Graphics2D) g).draw(this.thumbRect);
		// g.setColor(Color.red);
		// ((Graphics2D) g).draw(this.trackRect);
		// g.setColor(Color.green);
		// ((Graphics2D) g).draw(this.tickRect);
		// g.setColor(Color.magenta);
		// ((Graphics2D) g).draw(this.labelRect);
	}

	/**
	 * Returns the button model for tracking the thumb transitions.
	 * 
	 * @return Button model for tracking the thumb transitions.
	 */
	public ButtonModel getButtonModel() {
		return this.thumbModel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.substance.Trackable#isInside(java.awt.event.MouseEvent)
	 */
	public boolean isInside(MouseEvent me) {
		Rectangle thumbB = this.thumbRect;
		if (thumbB == null)
			return false;
		return thumbB.contains(me.getX(), me.getY());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#installListeners(javax.swing.JSlider)
	 */
	@Override
	protected void installListeners(final JSlider slider) {
		super.installListeners(slider);

		// fix for defect 109 - memory leak on changing theme
		this.substanceRolloverListener = new RolloverControlListener(this,
				this.thumbModel);
		slider.addMouseListener(this.substanceRolloverListener);
		slider.addMouseMotionListener(this.substanceRolloverListener);

		this.substancePropertyChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if ("enabled".equals(evt.getPropertyName())) {
					thumbModel.setEnabled(slider.isEnabled());
				}
			}
		};
		this.slider
				.addPropertyChangeListener(this.substancePropertyChangeListener);

		this.substanceFadeStateListener = new FadeStateListener(this.slider,
				this.thumbModel, null);
		this.substanceFadeStateListener.registerListeners(false);

		//		
		// this.substanceMouseMotionListener = new MouseMotionAdapter() {
		// @Override
		// public void mouseMoved(MouseEvent e) {
		// calculateThumbSize();
		// //System.out.println("-->" + thumbRect);
		// }
		// };
		// this.slider.addMouseMotionListener(this.substanceMouseMotionListener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#uninstallListeners(javax.swing.JSlider)
	 */
	@Override
	protected void uninstallListeners(JSlider slider) {
		super.uninstallListeners(slider);

		// fix for defect 109 - memory leak on changing theme
		slider.removeMouseListener(this.substanceRolloverListener);
		slider.removeMouseMotionListener(this.substanceRolloverListener);
		this.substanceRolloverListener = null;

		slider
				.removePropertyChangeListener(this.substancePropertyChangeListener);
		this.substancePropertyChangeListener = null;

		this.substanceFadeStateListener.unregisterListeners();
		this.substanceFadeStateListener = null;
		//		
		// slider.removeMouseMotionListener(this.substanceMouseMotionListener);
		// this.substanceMouseMotionListener = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#paintFocus(java.awt.Graphics)
	 */
	@Override
	public void paintFocus(Graphics g) {
		SubstanceCoreUtilities.paintFocus(g, this.slider, this.slider, null);
		// FadeTracker fadeTracker = FadeTracker.getInstance();
		// FocusKind focusKind =
		// SubstanceCoreUtilities.getFocusKind(this.slider);
		// if ((focusKind == FocusKind.NONE)
		// && (!fadeTracker.isTracked(this.slider, FadeKind.FOCUS)))
		// return;
		//
		// Graphics2D graphics = (Graphics2D) g.create();
		// graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
		// RenderingHints.VALUE_ANTIALIAS_ON);
		//
		// float alpha = 1.0f;
		// if (fadeTracker.isTracked(this.slider, FadeKind.FOCUS)) {
		// alpha = fadeTracker.getFade10(this.slider, FadeKind.FOCUS) / 10.f;
		// }
		// graphics.setComposite(TransitionLayout.getAlphaComposite(this.slider,
		// alpha));
		//
		// Color color = SubstanceColorUtilities.getFocusColor(this.slider);
		// graphics.setColor(color);
		// focusKind.paintFocus(this.slider, graphics, null);
		// graphics.dispose();
	}

	/**
	 * Returns the amount that the thumb goes past the slide bar.
	 * 
	 * @return Amount that the thumb goes past the slide bar.
	 */
	protected int getThumbOverhang() {
		return (int) (getThumbSize().getHeight() - getTrackWidth()) / 2;
	}

	/**
	 * Returns the shorter dimension of the track.
	 * 
	 * @return Shorter dimension of the track.
	 */
	protected int getTrackWidth() {
		return SubstanceSizeUtils.getSliderTrackSize();
	}

	public void paintTicks(Graphics g) {
		Rectangle tickBounds = tickRect;
		if (slider.getOrientation() == JSlider.HORIZONTAL) {
			g.translate(0, tickBounds.y);

			int value = slider.getMinimum() + slider.getMinorTickSpacing();
			int xPos = 0;

			if (slider.getMinorTickSpacing() > 0) {
				while (value < slider.getMaximum()) {
					int delta = value - slider.getMinimum();
					if (delta % slider.getMajorTickSpacing() != 0) {
						xPos = xPositionForValue(value);
						paintMinorTickForHorizSlider(g, tickBounds, xPos);
					}
					value += slider.getMinorTickSpacing();
				}
			}

			if (slider.getMajorTickSpacing() > 0) {
				value = slider.getMinimum() + slider.getMajorTickSpacing();

				while (value < slider.getMaximum()) {
					xPos = xPositionForValue(value);
					paintMajorTickForHorizSlider(g, tickBounds, xPos);
					value += slider.getMajorTickSpacing();
				}
			}

			g.translate(0, -tickBounds.y);
		} else {
			g.translate(tickBounds.x, 0);

			int value = slider.getMinimum() + slider.getMinorTickSpacing();
			int yPos = 0;

			boolean ltr = this.slider.getComponentOrientation().isLeftToRight();
			if (slider.getMinorTickSpacing() > 0) {
				int offset = 0;
				if (!ltr) {
					offset = tickBounds.width - tickBounds.width / 2;
					g.translate(offset, 0);
				}

				while (value < slider.getMaximum()) {
					yPos = yPositionForValue(value);
					paintMinorTickForVertSlider(g, tickBounds, yPos);
					value += slider.getMinorTickSpacing();
				}

				if (!ltr) {
					g.translate(-offset, 0);
				}
			}

			if (slider.getMajorTickSpacing() > 0) {
				value = slider.getMinimum() + slider.getMajorTickSpacing();
				if (!ltr) {
					g.translate(2, 0);
				}

				while (value < slider.getMaximum()) {
					yPos = yPositionForValue(value);
					paintMajorTickForVertSlider(g, tickBounds, yPos);
					value += slider.getMajorTickSpacing();
				}

				if (!ltr) {
					g.translate(-2, 0);
				}
			}
			g.translate(-tickBounds.x, 0);
		}
	}

	@Override
	protected void paintMajorTickForHorizSlider(Graphics g,
			Rectangle tickBounds, int x) {
		boolean isDark = SubstanceCoreUtilities
				.isThemeDark(SubstanceLookAndFeel.getTheme());
		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
				.getActiveScheme(this.slider)
				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
		Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
				.getDarkColor();
		Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
				.getUltraLightColor();

		Graphics2D graphics = (Graphics2D) g.create();
		graphics.setPaint(new GradientPaint(x - 1, 0, color, x - 1,
				tickBounds.height, SubstanceColorUtilities.getAlphaColor(color,
						96)));
		graphics.drawLine(x - 1, 0, x - 1, tickBounds.height - 1);
		graphics.setPaint(new GradientPaint(x, 0, icolor, x, tickBounds.height,
				SubstanceColorUtilities.getAlphaColor(icolor, 96)));
		graphics.drawLine(x, 0, x, tickBounds.height - 1);
		graphics.dispose();
	}

	@Override
	protected void paintMajorTickForVertSlider(Graphics g,
			Rectangle tickBounds, int y) {
		boolean isDark = SubstanceCoreUtilities
				.isThemeDark(SubstanceLookAndFeel.getTheme());
		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
				.getActiveScheme(this.slider)
				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
		Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
				.getDarkColor();
		Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
				.getUltraLightColor();

		Graphics2D graphics = (Graphics2D) g.create();
		if (this.slider.getComponentOrientation().isLeftToRight()) {
			graphics.setPaint(new GradientPaint(0, y, color,
					tickBounds.width - 1, y, SubstanceColorUtilities
							.getAlphaColor(color, 96)));
		} else {
			graphics.setPaint(new GradientPaint(0, y, SubstanceColorUtilities
					.getAlphaColor(color, 96), tickBounds.width - 1, y, color));
		}
		graphics.drawLine(0, y, tickBounds.width - 1, y);
		if (this.slider.getComponentOrientation().isLeftToRight()) {
			graphics.setPaint(new GradientPaint(0, y + 1, icolor,
					tickBounds.width - 1, y + 1, SubstanceColorUtilities
							.getAlphaColor(icolor, 96)));
		} else {
			graphics.setPaint(new GradientPaint(0, y + 1,
					SubstanceColorUtilities.getAlphaColor(icolor, 96),
					tickBounds.width - 1, y + 1, icolor));
		}
		graphics.drawLine(0, y + 1, tickBounds.width - 1, y + 1);
		graphics.dispose();
	}

	@Override
	protected void paintMinorTickForHorizSlider(Graphics g,
			Rectangle tickBounds, int x) {
		boolean isDark = SubstanceCoreUtilities
				.isThemeDark(SubstanceLookAndFeel.getTheme());
		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
				.getActiveScheme(this.slider)
				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
		Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
				.getDarkColor();
		Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
				.getUltraLightColor();

		Graphics2D graphics = (Graphics2D) g.create();
		graphics.setPaint(new GradientPaint(x - 1, 0, color, x - 1,
				tickBounds.height / 2, SubstanceColorUtilities.getAlphaColor(
						color, 96)));
		graphics.drawLine(x - 1, 0, x - 1, tickBounds.height / 2);
		graphics.setPaint(new GradientPaint(x, 0, icolor, x,
				tickBounds.height / 2, SubstanceColorUtilities.getAlphaColor(
						icolor, 96)));
		graphics.drawLine(x, 0, x, tickBounds.height / 2);
		graphics.dispose();
	}

	@Override
	protected void paintMinorTickForVertSlider(Graphics g,
			Rectangle tickBounds, int y) {
		boolean isDark = SubstanceCoreUtilities
				.isThemeDark(SubstanceLookAndFeel.getTheme());
		ColorScheme colorScheme = this.slider.isEnabled() ? SubstanceCoreUtilities
				.getActiveScheme(this.slider)
				: SubstanceCoreUtilities.getDisabledScheme(this.slider);
		Color color = isDark ? colorScheme.getUltraLightColor() : colorScheme
				.getDarkColor();
		Color icolor = isDark ? colorScheme.getDarkColor() : colorScheme
				.getUltraLightColor();

		Graphics2D graphics = (Graphics2D) g.create();
		if (this.slider.getComponentOrientation().isLeftToRight()) {
			graphics.setPaint(new GradientPaint(0, y, color,
					tickBounds.width / 2, y, SubstanceColorUtilities
							.getAlphaColor(color, 96)));
			graphics.drawLine(0, y, tickBounds.width / 2, y);
		} else {
			graphics.setPaint(new GradientPaint(1, y, SubstanceColorUtilities
					.getAlphaColor(color, 96), tickBounds.width / 2 + 1, y,
					color));
			graphics.drawLine(1, y, tickBounds.width / 2 + 1, y);
		}
		if (this.slider.getComponentOrientation().isLeftToRight()) {
			graphics.setPaint(new GradientPaint(0, y + 1, icolor,
					tickBounds.width / 2, y + 1, SubstanceColorUtilities
							.getAlphaColor(icolor, 96)));
			graphics.drawLine(0, y + 1, tickBounds.width / 2, y + 1);
		} else {
			graphics.setPaint(new GradientPaint(1, y + 1,
					SubstanceColorUtilities.getAlphaColor(icolor, 96),
					tickBounds.width / 2 + 1, y + 1, icolor));
			graphics.drawLine(1, y + 1, tickBounds.width / 2 + 1, y + 1);
		}
		graphics.dispose();
	}

	@Override
	protected void calculateTickRect() {
		if (slider.getOrientation() == JSlider.HORIZONTAL) {
			tickRect.x = trackRect.x;
			tickRect.y = trackRect.y + trackRect.height;
			tickRect.width = trackRect.width;
			tickRect.height = (slider.getPaintTicks()) ? getTickLength() : 0;
		} else {
			tickRect.width = (slider.getPaintTicks()) ? getTickLength() : 0;
			if (slider.getComponentOrientation().isLeftToRight()) {
				tickRect.x = trackRect.x + trackRect.width;
			} else {
				tickRect.x = trackRect.x - tickRect.width;
			}
			tickRect.y = trackRect.y;
			tickRect.height = trackRect.height;
		}

		if (slider.getPaintTicks()) {
			if (slider.getOrientation() == JSlider.HORIZONTAL) {
				tickRect.y -= 3;
			} else {
				if (slider.getComponentOrientation().isLeftToRight()) {
					tickRect.x -= 2;
				} else {
					tickRect.x += 2;
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#calculateLabelRect()
	 */
	@Override
	protected void calculateLabelRect() {
		super.calculateLabelRect();
		if (slider.getOrientation() == JSlider.HORIZONTAL) {
			if (!slider.getPaintTicks()) {
				// labelRect.y += 1;
			}
		} else {
			if (!slider.getPaintTicks()) {
				if (this.slider.getComponentOrientation().isLeftToRight()) {
					labelRect.x += 3;
				} else {
					// labelRect.x -= 2;
				}

			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#paintHorizontalLabel(java.awt.Graphics,
	 *      int, java.awt.Component)
	 */
	@Override
	protected void paintHorizontalLabel(Graphics g, int value, Component label) {
		label.setEnabled(this.slider.isEnabled());
		super.paintHorizontalLabel(g, value, label);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSliderUI#paintVerticalLabel(java.awt.Graphics,
	 *      int, java.awt.Component)
	 */
	@Override
	protected void paintVerticalLabel(Graphics g, int value, Component label) {
		label.setEnabled(this.slider.isEnabled());
		super.paintVerticalLabel(g, value, label);
	}

	public Dimension getPreferredSize(JComponent c) {
		recalculateIfInsetsChanged();
		Dimension d;
		if (slider.getOrientation() == JSlider.VERTICAL) {
			d = new Dimension(getPreferredVerticalSize());
			d.width = insetCache.left + insetCache.right;
			d.width += focusInsets.left + focusInsets.right;
			d.width += trackRect.width /* + tickRect.width */+ labelRect.width;
			// d.width -= 6;
			if (slider.getPaintTicks())
				d.width += 3;
			if (slider.getPaintLabels())
				d.width += 1;
			d.width += 3;
			// d.width += 6;
		} else {
			d = new Dimension(getPreferredHorizontalSize());
			d.height = insetCache.top + insetCache.bottom;
			d.height += focusInsets.top + focusInsets.bottom;
			d.height += trackRect.height /* + tickRect.height */
					+ labelRect.height;
			// d.height -= 6;
			if (slider.getPaintTicks())
				d.height += 3;
			if (slider.getPaintLabels())
				d.height += 1;
			d.height += 3;
		}

		return d;
	}
}
