/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance;

import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashSet;
import java.util.Set;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicSpinnerUI;

import org.jvnet.lafwidget.utils.FadeStateListener;
import org.jvnet.lafwidget.utils.FadeTracker;
import org.jvnet.lafwidget.utils.FadeTracker.FadeKind;
import org.jvnet.substance.utils.*;
import org.jvnet.substance.utils.SubstanceConstants.Side;

/**
 * UI for spinners in <b>Substance</b> look and feel.
 * 
 * @author Kirill Grouchnikov
 */
public class SubstanceSpinnerUI extends BasicSpinnerUI {
	// /**
	// * Focus listener.
	// */
	// protected FocusBorderListener substanceFocusListener;

	/**
	 * Tracks changes to editor, removing the border as necessary.
	 */
	protected PropertyChangeListener substancePropertyChangeListener;

	/**
	 * Listener for fade animations.
	 */
	protected FadeStateListener substanceFadeStateListener;

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#createUI(javax.swing.JComponent)
	 */
	public static ComponentUI createUI(JComponent c) {
		return new SubstanceSpinnerUI();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSpinnerUI#createNextButton()
	 */
	@Override
	protected Component createNextButton() {
		JComponent c = new SubstanceSpinnerButton(this.spinner,
				SubstanceImageCreator.getArrowIcon(SubstanceSizeUtils
						.getArrowIconWidth() - 2, SubstanceSizeUtils
						.getArrowIconHeight() - 1, SubstanceSizeUtils
						.getArrowStrokeWidth(), SwingConstants.NORTH,
						SubstanceCoreUtilities.getActiveTheme(this.spinner,
								true)), SwingConstants.NORTH);
		int spinnerButtonSize = SubstanceSizeUtils.getSpinnerButtonWidth();
		c.setPreferredSize(new Dimension(spinnerButtonSize, spinnerButtonSize));
		c.setMinimumSize(new Dimension(5, 5));

		Set<Side> openSides = new HashSet<Side>();
		openSides.add(Side.BOTTOM);
		c.putClientProperty(SubstanceLookAndFeel.BUTTON_OPEN_SIDE_PROPERTY,
				openSides);

		this.installNextButtonListeners(c);
		return c;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSpinnerUI#createPreviousButton()
	 */
	@Override
	protected Component createPreviousButton() {
		JComponent c = new SubstanceSpinnerButton(this.spinner,
				SubstanceImageCreator.getArrowIcon(SubstanceSizeUtils
						.getArrowIconWidth() - 2, SubstanceSizeUtils
						.getArrowIconHeight() - 1, SubstanceSizeUtils
						.getArrowStrokeWidth(), SwingConstants.SOUTH,
						SubstanceCoreUtilities.getActiveTheme(this.spinner,
								true)), SwingConstants.SOUTH);
		int spinnerButtonSize = SubstanceSizeUtils.getSpinnerButtonWidth();
		c.setPreferredSize(new Dimension(spinnerButtonSize, spinnerButtonSize));
		c.setMinimumSize(new Dimension(5, 5));

		Set<Side> openSides = new HashSet<Side>();
		openSides.add(Side.TOP);
		c.putClientProperty(SubstanceLookAndFeel.BUTTON_OPEN_SIDE_PROPERTY,
				openSides);

		this.installPreviousButtonListeners(c);
		return c;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSpinnerUI#installDefaults()
	 */
	@Override
	protected void installDefaults() {
		super.installDefaults();
		JComponent editor = this.spinner.getEditor();
		if ((editor != null) && (editor instanceof JSpinner.DefaultEditor)) {
			JTextField tf = ((JSpinner.DefaultEditor) editor).getTextField();
			if (tf != null) {
				tf.setBorder(new EmptyBorder(0, 1, 0, 1));
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSpinnerUI#installListeners()
	 */
	@Override
	protected void installListeners() {
		super.installListeners();
		// this.substanceFocusListener = new FocusBorderListener(this.spinner);
		// this.spinner.addFocusListener(this.substanceFocusListener);

		this.substancePropertyChangeListener = new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				if ("editor".equals(evt.getPropertyName())) {
					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							if (substanceFadeStateListener != null) {
								substanceFadeStateListener
										.unregisterListeners();
							}
							if (spinner == null)
								return;
							JComponent editor = spinner.getEditor();
							if ((editor != null)
									&& (editor instanceof JSpinner.DefaultEditor)) {
								JTextField tf = ((JSpinner.DefaultEditor) editor)
										.getTextField();
								if (tf != null) {
									tf.setBorder(new EmptyBorder(0, 1, 0, 1));
									substanceFadeStateListener = new FadeStateListener(
											tf,
											null,
											new FadeTracker.FadeTrackerCallback() {
												public void fadeEnded(
														FadeKind fadeKind) {
													if (spinner != null)
														spinner.repaint();
												}

												public void fadePerformed(
														FadeKind fadeKind,
														float fadeCycle10) {
													if (spinner != null)
														spinner.repaint();
												}
											});
									substanceFadeStateListener
											.registerListeners();
								}
							}
						}
					});
				}
			}
		};
		this.spinner
				.addPropertyChangeListener(this.substancePropertyChangeListener);

		JComponent editor = spinner.getEditor();
		if ((editor != null) && (editor instanceof JSpinner.DefaultEditor)) {
			JTextField tf = ((JSpinner.DefaultEditor) editor).getTextField();
			this.substanceFadeStateListener = new FadeStateListener(tf, null,
					new FadeTracker.FadeTrackerCallback() {
						public void fadeEnded(FadeKind fadeKind) {
							if (spinner != null)
								spinner.repaint();
						}

						public void fadePerformed(FadeKind fadeKind,
								float fadeCycle10) {
							if (spinner != null)
								spinner.repaint();
						}
					});
			this.substanceFadeStateListener.registerListeners();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSpinnerUI#uninstallListeners()
	 */
	@Override
	protected void uninstallListeners() {
		this.substanceFadeStateListener.unregisterListeners();
		this.substanceFadeStateListener = null;

		// this.substanceFocusListener.cancelAnimations();
		// this.spinner.removeFocusListener(this.substanceFocusListener);
		// this.substanceFocusListener = null;

		this.spinner
				.removePropertyChangeListener(this.substancePropertyChangeListener);
		this.substancePropertyChangeListener = null;

		super.uninstallListeners();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.ComponentUI#paint(java.awt.Graphics,
	 *      javax.swing.JComponent)
	 */
	@Override
	public void paint(Graphics g, JComponent c) {
		super.paint(g, c);
		if (hasFocus(this.spinner)
				|| FadeTracker.getInstance().isTracked(this.spinner,
						FadeKind.FOCUS)) {
			this.paintFocus(g, this.spinner.getEditor().getBounds());
		}
	}

	/**
	 * Checks if a component or any of its children have focus.
	 * 
	 * @param comp
	 *            Component.
	 * @return <code>true</code> if the component of any of its children have
	 *         focus, <code>false</code> otherwise.
	 */
	private static boolean hasFocus(Component comp) {
		if (comp.hasFocus())
			return true;
		if (comp instanceof Container) {
			Container cont = (Container) comp;
			for (int i = 0; i < cont.getComponentCount(); i++) {
				Component child = cont.getComponent(i);
				if (hasFocus(child))
					return true;
			}
		}
		return false;
	}

	/**
	 * Paints the focus indication.
	 * 
	 * @param g
	 *            Graphics.
	 * @param bounds
	 *            Bounds for text.
	 */
	protected void paintFocus(Graphics g, Rectangle bounds) {
		// FadeTracker fadeTracker = FadeTracker.getInstance();
		JComponent editor = spinner.getEditor();
		if ((editor != null) && (editor instanceof JSpinner.DefaultEditor)) {
			JTextField tf = ((JSpinner.DefaultEditor) editor).getTextField();
			if (tf != null) {
				SubstanceCoreUtilities.paintFocus(g, this.spinner, tf, bounds,
						0.4f, 2 + SubstanceSizeUtils.getExtraPadding());
				//				
				// FocusKind focusKind = SubstanceCoreUtilities
				// .getFocusKind(this.spinner);
				// if ((focusKind == FocusKind.NONE)
				// && (!fadeTracker.isTracked(tf, FadeKind.FOCUS)))
				// return;
				//
				// Graphics2D graphics = (Graphics2D) g.create();
				// graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				// RenderingHints.VALUE_ANTIALIAS_ON);
				//
				// float alpha = 1.0f;
				// if (fadeTracker.isTracked(tf, FadeKind.FOCUS)) {
				// alpha = fadeTracker.getFade10(tf, FadeKind.FOCUS) / 10.f;
				// }
				// graphics.setComposite(TransitionLayout.getAlphaComposite(
				// this.spinner, alpha));
				//
				// Color color = SubstanceColorUtilities
				// .getFocusColor(this.spinner);
				// graphics.setColor(color);
				// focusKind.paintFocus(this.spinner, tf, graphics, bounds);
				// graphics.dispose();
			}
		}
	}
}
