/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance;

import java.awt.*;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicToolBarSeparatorUI;

import org.jvnet.substance.color.ColorScheme;
import org.jvnet.substance.theme.SubstanceTheme;
import org.jvnet.substance.title.SubstanceHeaderPainter;
import org.jvnet.substance.title.SubstanceTitlePainter;
import org.jvnet.substance.utils.SubstanceCoreUtilities;

/**
 * UI for toolbar separators in <b>Substance</b> look and feel.
 * 
 * @author Kirill Grouchnikov
 */
public class SubstanceToolBarSeparatorUI extends BasicToolBarSeparatorUI {
	public static ComponentUI createUI(JComponent c) {
		return new SubstanceToolBarSeparatorUI();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSeparatorUI#paint(java.awt.Graphics,
	 *      javax.swing.JComponent)
	 */
	@Override
	public void paint(Graphics g, JComponent c) {
		Graphics2D graphics = (Graphics2D) g.create();
		// Dimension s = c.getSize();
		SubstanceTitlePainter titlePainter = SubstanceCoreUtilities
				.getTitlePainter(SwingUtilities.getRootPane(c));
		boolean shouldUseHeaderPainter = titlePainter instanceof SubstanceHeaderPainter;
		if (titlePainter instanceof SubstanceHeaderPainter)
			shouldUseHeaderPainter = ((SubstanceHeaderPainter) titlePainter)
					.isPaintingToolbars();
		SubstanceTheme theme = shouldUseHeaderPainter ? SubstanceLookAndFeel
				.getTheme().getActiveTitlePaneTheme() : SubstanceLookAndFeel
				.getTheme();
		boolean isDark = SubstanceCoreUtilities.isThemeDark(theme
				.getActiveTheme());
		ColorScheme colorScheme = (shouldUseHeaderPainter || isDark) ? theme
				.getActiveTheme().getColorScheme() : theme.getDefaultTheme()
				.getColorScheme();

		SubstanceCoreUtilities.paintSeparator(graphics, colorScheme, isDark, c
				.getWidth(), c.getHeight(), ((JSeparator) c).getOrientation());
		//
		// Color foreLight = isDark ? colorScheme.getLightColor()
		// : SubstanceColorUtilities.getInterpolatedColor(colorScheme
		// .getLightColor(), colorScheme.getDarkColor(), 0.8);
		// Color foreDark = isDark ? colorScheme.getExtraLightColor()
		// : SubstanceColorUtilities.getInterpolatedColor(colorScheme
		// .getMidColor(), colorScheme.getDarkColor(), 0.4);
		// Color back = isDark ? colorScheme.getDarkColor() : colorScheme
		// .getUltraLightColor();
		//
		// if (((JSeparator) c).getOrientation() == JSeparator.VERTICAL) {
		// graphics.translate(Math.max(0, s.width / 2 - 1), 0);
		// graphics.setPaint(new GradientPaint(0, 0, SubstanceColorUtilities
		// .getAlphaColor(foreLight, 32), 0, s.height / 2,
		// SubstanceColorUtilities.getAlphaColor(foreDark, 240)));
		// graphics.drawLine(0, 0, 0, s.height / 2);
		// graphics.setPaint(new GradientPaint(0, s.height / 2,
		// SubstanceColorUtilities.getAlphaColor(foreDark, 240), 0,
		// s.height, SubstanceColorUtilities.getAlphaColor(foreLight,
		// 32)));
		// graphics.drawLine(0, s.height / 2, 0, s.height);
		//
		// graphics.setPaint(new GradientPaint(1, 0, SubstanceColorUtilities
		// .getAlphaColor(back, 32), 1, s.height / 2,
		// SubstanceColorUtilities.getAlphaColor(back, 196)));
		// graphics.drawLine(1, 0, 1, s.height / 2);
		// graphics.setPaint(new GradientPaint(1, s.height / 2,
		// SubstanceColorUtilities.getAlphaColor(back, 196), 1,
		// s.height, SubstanceColorUtilities.getAlphaColor(back, 32)));
		// graphics.drawLine(1, s.height / 2, 1, s.height);
		// } else // HORIZONTAL
		// {
		// graphics.translate(0, Math.max(0, s.height / 2 - 1));
		// graphics.setPaint(new GradientPaint(0, 0, SubstanceColorUtilities
		// .getAlphaColor(foreLight, 32), s.width / 2, 0,
		// SubstanceColorUtilities.getAlphaColor(foreDark, 240)));
		// graphics.drawLine(0, 0, s.width / 2, 0);
		// graphics.setPaint(new GradientPaint(s.width / 2, 0,
		// SubstanceColorUtilities.getAlphaColor(foreDark, 240),
		// s.width, 0, SubstanceColorUtilities.getAlphaColor(
		// foreLight, 32)));
		// graphics.drawLine(s.width / 2, 0, s.width, 0);
		//
		// graphics.setPaint(new GradientPaint(0, 1, SubstanceColorUtilities
		// .getAlphaColor(back, 32), s.width / 2, 1,
		// SubstanceColorUtilities.getAlphaColor(back, 196)));
		// graphics.drawLine(0, 1, s.width / 2, 1);
		// graphics.setPaint(new GradientPaint(s.width / 2, 1,
		// SubstanceColorUtilities.getAlphaColor(back, 196), s.width,
		// 1, SubstanceColorUtilities.getAlphaColor(back, 32)));
		// graphics.drawLine(s.width / 2, 1, s.width, 1);
		// }
		graphics.dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicToolBarSeparatorUI#getPreferredSize(javax.swing.JComponent)
	 */
	@Override
	public Dimension getPreferredSize(JComponent c) {
		Dimension size = ((JToolBar.Separator) c).getSeparatorSize();

		if (size != null) {
			size = size.getSize();
		} else {
			size = new Dimension(6, 6);

			if (((JSeparator) c).getOrientation() == SwingConstants.VERTICAL) {
				size.height = 0;
			} else {
				size.width = 0;
			}
		}
		return size;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.plaf.basic.BasicSeparatorUI#getMaximumSize(javax.swing.JComponent)
	 */
	public Dimension getMaximumSize(JComponent c) {
		Dimension pref = getPreferredSize(c);
		if (((JSeparator) c).getOrientation() == SwingConstants.VERTICAL) {
			return new Dimension(pref.width, Short.MAX_VALUE);
		} else {
			return new Dimension(Short.MAX_VALUE, pref.height);
		}
	}

}
