/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance.combo;

import java.awt.Color;
import java.awt.Component;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

/**
 * Renderer for combo boxes.
 * 
 * @author Kirill Grouchnikov
 */
public class SubstanceComboBoxRenderer extends BasicComboBoxRenderer {
	/**
	 * The associated combo box.
	 */
	private JComboBox combo;

	/**
	 * Simple constructor.
	 * 
	 * @param combo
	 *            The associated combo box.
	 */
	public SubstanceComboBoxRenderer(JComboBox combo) {
		super();
		this.combo = combo;
		this.setOpaque(true);
		this.setBorder(new EmptyBorder(0, 2, 0, 2));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.ListCellRenderer#getListCellRendererComponent(javax.swing.JList,
	 *      java.lang.Object, int, boolean, boolean)
	 */
	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {

		if (isSelected) {
			this.setForeground(list.getSelectionForeground());
		} else {
			if (index % 2 == 0)
				this.setBackground(this.combo.getBackground());
			else {
				int r = this.combo.getBackground().getRed();
				int g = this.combo.getBackground().getGreen();
				int b = this.combo.getBackground().getBlue();
				this.setBackground(new Color((int) (0.96 * r),
						(int) (0.96 * g), (int) (0.96 * b)));
			}
			this.setForeground(this.combo.getForeground());
		}

		this.setFont(list.getFont());

		if (value instanceof Icon) {
			this.setIcon((Icon) value);
		} else {
			this.setText((value == null) ? "" : value.toString());
		}

		// for gradient background on selected item
		this.setOpaque(!isSelected && (index >= 0));
		return this;
	}

	/**
	 * UI resource for renderer (does nothing yet).
	 * 
	 * @author Kirill Grouchnikov
	 */
	public static class SubstanceUIResource extends SubstanceComboBoxRenderer
			implements javax.swing.plaf.UIResource {
		/**
		 * Creates a new renderer resource.
		 * 
		 * @param combo
		 *            Combobox.
		 */
		public SubstanceUIResource(JComboBox combo) {
			super(combo);
		}
	}
}
