/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance.skin;

import org.jvnet.substance.button.ClassicButtonShaper;
import org.jvnet.substance.theme.*;
import org.jvnet.substance.theme.SubstanceTheme.ThemeKind;
import org.jvnet.substance.watermark.SubstanceMetalWallWatermark;

/**
 * <code>Streetlights</code> skin. This class is <b>for internal use only</b>.
 * 
 * @author Kirill Grouchnikov
 */
public class StreetlightsSkin extends SubstanceAbstractSkin {
	/**
	 * Creates a new <code>Streetlights</code> skin.
	 */
	public StreetlightsSkin() {
		SubstanceTheme activeTheme = new SubstanceSunsetTheme().saturate(0.4);
		SubstanceTheme defaultTheme = new SubstanceEbonyTheme();
		SubstanceTheme disabledTheme = new SubstanceMixTheme(
				new SubstanceBlendBiTheme(new SubstanceBottleGreenTheme(),
						new SubstanceSteelBlueTheme(), 0.8).shade(0.6),
				new SubstanceBlendBiTheme(new SubstanceAquaTheme(),
						new SubstanceSteelBlueTheme(), 0.8).shade(0.6));
		SubstanceTheme activeTitleTheme = defaultTheme.saturate(0.2);
		SubstanceTheme watermarkTheme = new SubstanceBottleGreenTheme()
				.shade(0.5);

		this.theme = new SubstanceComplexTheme("Streetlights", ThemeKind.DARK,
				activeTheme, defaultTheme, disabledTheme, activeTitleTheme,
				watermarkTheme);
		this.shaper = new ClassicButtonShaper();
		this.watermark = new SubstanceMetalWallWatermark();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.substance.skin.SubstanceSkin#getDisplayName()
	 */
	public String getDisplayName() {
		return "Streetlights";
	}
}
