/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance.utils;

import java.awt.Color;

import javax.swing.*;

import org.jvnet.substance.title.SubstanceHeaderPainter;
import org.jvnet.substance.title.SubstanceTitlePainter;

/**
 * Helper extension of {@link Color} class that is based on the current visual
 * status of a delegate menu.
 * 
 * @author Kirill Grouchnikov
 */
public class MenuColorDelegate extends AbstractColorDelegate {
	/**
	 * The delegate menu item.
	 */
	protected JMenuItem menuItem;

	/**
	 * Menu color when the menu is in the header.
	 */
	protected Color colorInHeader;

	/**
	 * Menu color when the menu is not in the header.
	 */
	protected Color colorNotInHeader;

	/**
	 * Creates a new color delegate.
	 * 
	 * @param menu
	 *            The delegate menu item.
	 * @param colorInHeader
	 *            Menu color when the menu is in the header.
	 * @param colorNotInHeader
	 *            Menu color when the menu is not in the header.
	 */
	public MenuColorDelegate(JMenuItem menu, Color colorInHeader,
			Color colorNotInHeader) {
		super(colorInHeader.getRGB());
		this.menuItem = menu;
		this.colorInHeader = colorInHeader;
		this.colorNotInHeader = colorNotInHeader;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.jvnet.substance.utils.AbstractColorDelegate#getDelegate()
	 */
	protected Color getDelegate() {
		SubstanceTitlePainter titlePainter = SubstanceCoreUtilities
				.getTitlePainter(SwingUtilities.getRootPane(this.menuItem));

		if ((titlePainter instanceof SubstanceHeaderPainter)
				&& (this.menuItem.getParent() instanceof JMenuBar)) {
			// System.out.println(this.menuItem.getText() + " : header");
			return this.colorInHeader;
		} else {
			// System.out.println(this.menuItem.getText() + " : not header");
			return this.colorNotInHeader;
		}
	}
}
