/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package org.jvnet.substance.utils;

import java.awt.Insets;

import javax.swing.plaf.BorderUIResource;
import javax.swing.plaf.basic.BasicBorders;

import org.jvnet.substance.SubstanceBorder;
import org.jvnet.substance.SubstanceLookAndFeel;

import contrib.com.jgoodies.looks.common.FontPolicy;
import contrib.com.jgoodies.looks.common.FontSet;

public class SubstanceSizeUtils {
	private static int controlFontSize = -1;

	public static void setControlFontSize(int size) {
		controlFontSize = size;
	}

	public static int getControlFontSize() {
		if (controlFontSize > 0)
			return controlFontSize;
		FontPolicy fPolicy = SubstanceLookAndFeel.getFontPolicy();
		FontSet fSet = fPolicy.getFontSet("Substance", null);
		controlFontSize = fSet.getControlFont().getSize();
		return controlFontSize;
	}

	public static int getScrollBarWidth() {
		return getControlFontSize() + 4;
	}

	public static int getArrowIconWidth() {
		int result = getControlFontSize() - 2;
		if (result % 2 == 0)
			result--;
		return result;
	}

	public static int getArrowIconHeight() {
		return 6 + (getControlFontSize() - 11) / 2;
	}

	public static int getTabCloseIconSize() {
		return 8 + (getControlFontSize() - 6) / 2;
	}

	public static int getTitlePaneIconSize() {
		return 5 + getControlFontSize();
	}

	public static int getSliderIconSize() {
		int result = getControlFontSize() + 5;
		if (result % 2 == 1)
			result--;
		return result;
	}

	public static int getMenuCheckMarkSize() {
		int result = getControlFontSize() - 2;
		if (result % 2 == 0)
			result--;
		return result;
	}

	public static int getCheckBoxMarkSize() {
		return 5 + getControlFontSize();
	}

	public static int getRadioButtonMarkSize() {
		int result = getControlFontSize();
		if (result % 2 == 0)
			result--;
		return result;
	}

	public static int getSliderTrackSize() {
		return Math.max(4, 5 + (getControlFontSize() - 11) / 2);
	}

	public static float getArrowStrokeWidth() {
		return getControlFontSize() / 6.0f;
	}

	public static float getDoubleArrowStrokeWidth() {
		return getControlFontSize() / 8.0f;
	}

	public static float getTabCloseButtonStrokeWidth() {
		return getControlFontSize() / 10.0f;
	}

	public static int getSpinnerButtonWidth() {
		return getArrowIconWidth() * 3 / 2;
	}

	public static int getPasswordDotDiameter() {
		int result = getControlFontSize() - 5;
		if (result % 2 == 0)
			result++;
		return result;
	}

	public static int getPasswordDotGap() {
		return (getControlFontSize() - 6) / 3;
	}

	public static int getMinButtonWidth() {
		// return 3 * (14 + getControlFontSize()) - 5;
		return (int) (3.5 * getMinButtonHeight());
	}

	public static int getMinButtonHeight() {
		return 11 + (getControlFontSize() - 5) * 3 / 2;
	}

	public static int getExtraPadding() {
		int cfs = getControlFontSize();
		if (cfs < 14)
			return 0;
		return (cfs - 11) / 3;
	}

	public static int getExtraPadding(int baseSize) {
		int cfs = getControlFontSize();
		if (cfs < 14)
			return 0;
		return (cfs - 11) / (6 - baseSize);
	}

	public static Insets getTextBorderInsets() {
		int borderPadding = getExtraPadding();
		return new Insets(3 + borderPadding, 4 + borderPadding,
				3 + borderPadding, 4 + borderPadding);
	}

	public static Insets getComboBorderInsets() {
		int borderPadding = getExtraPadding();
		return new Insets(1 + borderPadding, 2 + borderPadding,
				1 + borderPadding, 2 + borderPadding);
	}

	public static Insets getSpinnerArrowButtonInsets() {
		int borderPadding = getExtraPadding();
		return new Insets(2 + borderPadding, borderPadding, 2 + borderPadding,
				2 + borderPadding);
	}

	public static Insets getSpinnerBorderInsets() {
		int borderPadding = getExtraPadding();
		return new Insets(3 + borderPadding, 3 + borderPadding,
				3 + borderPadding, 3 + borderPadding);
	}

	public static float getFocusStrokeWidth() {
		return getControlFontSize() / 10.0f;
	}
}
