/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.check;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.jvnet.lafwidget.LafWidget;
import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.combo.ComboPopupPrototypeCallback;
import org.jvnet.substance.combo.WidestComboPopupPrototype;

import test.Check;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

public class CombosPanel extends ControllablePanel {
	public static interface ComboCommand {
		public void invoke(JComboBox combo);
	}

	public static class EnableCommand implements ComboCommand {
		public void invoke(JComboBox combo) {
			combo.setEnabled(true);
		}
	}

	public static class DisableCommand implements ComboCommand {
		public void invoke(JComboBox combo) {
			combo.setEnabled(false);
		}
	}

	public static class EditAllowCommand implements ComboCommand {
		public void invoke(JComboBox combo) {
			combo.setEditable(true);
		}
	}

	public static class EditDisallowCommand implements ComboCommand {
		public void invoke(JComboBox combo) {
			combo.setEditable(false);
		}
	}

	public static void run(Component comp, ComboCommand command) {
		if (comp instanceof JComboBox) {
			command.invoke((JComboBox) comp);
			return;
		}
		if (comp instanceof Container) {
			Container cont = (Container) comp;
			for (int i = 0; i < cont.getComponentCount(); i++) {
				run(cont.getComponent(i), command);
			}
		}
	}

	private class ComboBoxTextEditor151 extends JTextField implements
			ComboBoxEditor {
		javax.swing.text.Document doc;

		public ComboBoxTextEditor151() {
			super();
			doc = super.getDocument();
		}

		public java.awt.Component getEditorComponent() {
			return this;
		}

		public void setItem(Object anObject) {
			if (anObject instanceof EditableDocument151) {
				super.setCaretPosition(0);
				super.setDocument((javax.swing.text.Document) anObject);
			} else {
				super.setCaretPosition(0);
				super.setDocument(doc);
				super.setText(anObject.toString());

			}

		}

		public Object getItem() {
			return super.getText();
		}

		public String toString() {
			return super.getText();
		}

	}

	private class EditableDocument151 extends javax.swing.text.PlainDocument {
		public EditableDocument151() {
			this.setText("A long text just to show the bug");
		}

		public String toString() {
			try {
				return super.getText(0, super.getLength());
			} catch (javax.swing.text.BadLocationException ex) {
				return "bad location";
			}
		}

		public void setText(String newText) {
			try {
				super.remove(0, super.getLength());
				super.insertString(0, newText,
						new javax.swing.text.SimpleAttributeSet());

			} catch (javax.swing.text.BadLocationException ex) {
				ex.printStackTrace();
			}

		}
	}

	private JPanel getLeftComboPanel() {
		FormLayout lm = new FormLayout("right:pref, 4dlu, fill:pref:grow", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
		builder.setDefaultDialogBorder();

		builder.appendSeparator("Simple combos");

		JComboBox comboRegular = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboRegular.setToolTipText("This is my combo 1");
		comboRegular.setMaximumRowCount(4);
		builder.append("Regular", comboRegular);

		JComboBox comboActive = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboActive.setToolTipText("This combo is active");
		comboActive.setMaximumRowCount(4);
		comboActive.putClientProperty(
				SubstanceLookAndFeel.PAINT_ACTIVE_PROPERTY, Boolean.TRUE);
		builder.append("Active", comboActive);

		JComboBox comboDisabled = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3" });
		comboDisabled.setEnabled(false);
		builder.append("Disabled", comboDisabled);

		JComboBox comboColored = new JComboBox(new Object[] { "entry31",
				"entry32", "entry33", "entry34", "entry35", "entry36", "aaa",
				"abb", "abc" });
		comboColored.setBackground(new Color(255, 128, 128));
		comboColored.setForeground(new Color(0, 0, 128));
		builder.append("Colored", comboColored);

		builder.appendSeparator("Auto-completion");

		final JComboBox comboCompletionTest = new JComboBox(new Object[] {
				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
		comboCompletionTest.setEditable(true);
		comboCompletionTest.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Check.out("'" + comboCompletionTest.getSelectedItem()
						+ "' selected");
			}
		});
		builder.append("Free w/ Object[]", comboCompletionTest);

		final JComboBox comboCompletionTest2 = new JComboBox();
		comboCompletionTest2.setEditable(true);
		comboCompletionTest2.setModel(new DefaultComboBoxModel(new Object[] {
				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" }));
		comboCompletionTest2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Check.out("'" + comboCompletionTest2.getSelectedItem()
						+ "' selected");
			}
		});
		builder.append("Free w/ default", comboCompletionTest2);

		final JComboBox comboCompletionModelOnlyTest = new JComboBox(
				new Object[] { "Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
		comboCompletionModelOnlyTest.setEditable(true);
		comboCompletionModelOnlyTest.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Check.out("'" + comboCompletionModelOnlyTest.getSelectedItem()
						+ "' selected");
			}
		});
		comboCompletionModelOnlyTest.putClientProperty(
				LafWidget.COMBO_BOX_USE_MODEL_ONLY, Boolean.TRUE);
		builder.append("Model-based", comboCompletionModelOnlyTest);

		final JComboBox comboNoAutoCompletion = new JComboBox(new Object[] {
				"Ester", "Jordi", "Jordina", "Jorge", "Sergi" });
		comboNoAutoCompletion.setEditable(true);
		comboNoAutoCompletion.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Check.out("'" + comboNoAutoCompletion.getSelectedItem()
						+ "' selected");
			}
		});
		comboNoAutoCompletion.putClientProperty(
				LafWidget.COMBO_BOX_NO_AUTOCOMPLETION, Boolean.TRUE);
		builder.append("No completion", comboNoAutoCompletion);

		builder.appendSeparator("Miscellaneous");

		JComboBox colorCombo = new ColorComboBox();
		builder.append("Color chooser", colorCombo);

		JComboBox comboHebrew = new JComboBox(new Object[] {
				"\u05e8\u05d0\u05e9\u05d9 1", "\u05e8\u05d0\u05e9\u05d9 2",
				"\u05e8\u05d0\u05e9\u05d9 3", "\u05e8\u05d0\u05e9\u05d9 4",
				"\u05e8\u05d0\u05e9\u05d9 5", "\u05e8\u05d0\u05e9\u05d9 6",
				"\u05e8\u05d0\u05e9\u05d9 7", "\u05e8\u05d0\u05e9\u05d9 8",
				"\u05e8\u05d0\u05e9\u05d9 9" });
		comboHebrew.setToolTipText("RTL combo");
		comboHebrew.setComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
		comboHebrew.setMaximumRowCount(6);
		builder.append("RTL (Hebrew)", comboHebrew);

		final JComboBox combo151 = new JComboBox();
		final ComboBoxTextEditor151 editor = new ComboBoxTextEditor151();
		combo151.setEditor(editor);
		combo151.addItem(new EditableDocument151());
		combo151.addItem("Default");
		combo151.addItem("No Sound");

		combo151.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(java.awt.event.ActionEvent e) {
				editor.setCaretPosition(0);
				if (combo151.getSelectedItem() instanceof String) {
					combo151.setEditable(false);

				} else {
					combo151.setEditable(true);

				}
				editor.setCaretPosition(0);
			}
		});
		builder.append("Defect 151", combo151);

		return builder.getPanel();
	}

	private JPanel getRightComboPanel() {
		FormLayout lm = new FormLayout("right:pref, 4dlu, left:pref:grow", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
		builder.setDefaultDialogBorder();

		builder.appendSeparator("Popup flyout");

		JComboBox comboFlyoutDefault = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboFlyoutDefault.setMaximumRowCount(4);
		builder.append("Default", comboFlyoutDefault);

		JComboBox comboFlyoutNorth = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboFlyoutNorth.setMaximumRowCount(4);
		comboFlyoutNorth.putClientProperty(
				SubstanceLookAndFeel.COMBO_BOX_POPUP_FLYOUT_ORIENTATION,
				SwingConstants.NORTH);
		builder.append("North", comboFlyoutNorth);

		JComboBox comboFlyoutEast = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboFlyoutEast.setMaximumRowCount(4);
		comboFlyoutEast.putClientProperty(
				SubstanceLookAndFeel.COMBO_BOX_POPUP_FLYOUT_ORIENTATION,
				SwingConstants.EAST);
		builder.append("East", comboFlyoutEast);

		JComboBox comboFlyoutSouth = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboFlyoutSouth.setMaximumRowCount(4);
		comboFlyoutSouth.putClientProperty(
				SubstanceLookAndFeel.COMBO_BOX_POPUP_FLYOUT_ORIENTATION,
				SwingConstants.SOUTH);
		builder.append("South", comboFlyoutSouth);

		JComboBox comboFlyoutWest = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboFlyoutWest.setMaximumRowCount(4);
		comboFlyoutWest.putClientProperty(
				SubstanceLookAndFeel.COMBO_BOX_POPUP_FLYOUT_ORIENTATION,
				SwingConstants.WEST);
		builder.append("West", comboFlyoutWest);

		JComboBox comboFlyoutCenter = new JComboBox(new Object[] { "entry1",
				"entry2", "entry3", "entry4", "entry5", "entry6" });
		comboFlyoutCenter.setMaximumRowCount(4);
		comboFlyoutCenter.putClientProperty(
				SubstanceLookAndFeel.COMBO_BOX_POPUP_FLYOUT_ORIENTATION,
				SwingConstants.CENTER);
		builder.append("Center", comboFlyoutCenter);

		builder.appendSeparator("Popup prototype");
		JComboBox comboProto1 = new JComboBox(new Object[] { "aa", "aaaaa",
				"aaaaaaaaaa", "this one is the one",
				"abcdefghijklmnopqrstuvwxyz" });
		comboProto1.setPrototypeDisplayValue("aaaaa");
		comboProto1.putClientProperty(
				SubstanceLookAndFeel.COMBO_POPUP_PROTOTYPE,
				"this one is the one");
		builder.append("Hard-coded value", comboProto1);

		JComboBox comboProto2 = new JComboBox(new Object[] { "aa", "aaaaa",
				"aaaaaaaaaa", "another one (not it)",
				"abcdefghijklmnopqrstuvwxyz" });
		comboProto2.setPrototypeDisplayValue("aaaaa");
		comboProto2.putClientProperty(
				SubstanceLookAndFeel.COMBO_POPUP_PROTOTYPE,
				new WidestComboPopupPrototype());
		builder.append("Widest core callback", comboProto2);

		JComboBox comboProto3 = new JComboBox(new Object[] { "aa", "aaaaa",
				"this is not", "this one is not it",
				"this one is it that is for the popup" });
		comboProto3.setPrototypeDisplayValue("aaaaa");
		comboProto3.putClientProperty(
				SubstanceLookAndFeel.COMBO_POPUP_PROTOTYPE,
				new ComboPopupPrototypeCallback() {
					public Object getPopupPrototypeDisplayValue(JComboBox jc) {
						return jc.getModel().getElementAt(
								jc.getModel().getSize() - 1);
					}
				});
		builder.append("Custom callback", comboProto3);

		return builder.getPanel();
	}

	public CombosPanel() {
		this.setLayout(new BorderLayout());

		JPanel gridPanel = new JPanel(new GridLayout(1, 2));
		gridPanel.add(getLeftComboPanel());
		gridPanel.add(getRightComboPanel());
		this.add(gridPanel, BorderLayout.CENTER);

		FormLayout lm = new FormLayout("fill:pref:grow", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);

		JButton checkEscape = new JButton("Check ESC key");
		checkEscape.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final JDialog dialog = new JDialog();
				dialog.setTitle("Press ESC key when combo is focused");
				dialog.setLayout(new FlowLayout());
				JComboBox sampleCombo = new JComboBox(new Object[] { "Ester",
						"Jordi", "Jordina", "Jorge", "Sergi" });
				sampleCombo.setEditable(true);
				dialog.add(sampleCombo);
				dialog.add(new JCheckBox("Sample checkbox"));
				dialog.add(new JButton("Sample button"));
				dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

				JRootPane rootPane = dialog.getRootPane();
				InputMap iMap = rootPane
						.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
				iMap.put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0),
						"escape");

				ActionMap aMap = rootPane.getActionMap();
				aMap.put("escape", new AbstractAction() {
					public void actionPerformed(ActionEvent e) {
						dialog.dispose();
					}
				});
				dialog.pack();
				Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
				// center the dialog in the physical screen
				dialog.setLocation((d.width - dialog.getWidth()) / 2,
						(d.height - dialog.getHeight()) / 2);
				dialog.setVisible(true);
			}
		});

		JButton disableCombosButton = new JButton("Disable all");
		disableCombosButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						CombosPanel.run(CombosPanel.this, new DisableCommand());
					}
				});
			}
		});
		JButton enableCombosButton = new JButton("Enable all");
		enableCombosButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						CombosPanel.run(CombosPanel.this, new EnableCommand());
					}
				});
			}
		});
		JButton makeAllEditableButton = new JButton("Make all editable");
		makeAllEditableButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						CombosPanel.run(CombosPanel.this,
								new EditAllowCommand());
					}
				});
			}
		});
		JButton makeAllNonEditableButton = new JButton("Make all non-editable");
		makeAllNonEditableButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						CombosPanel.run(CombosPanel.this,
								new EditDisallowCommand());
					}
				});
			}
		});

		builder.append(disableCombosButton);
		builder.append(enableCombosButton);
		builder.append(makeAllNonEditableButton);
		builder.append(makeAllEditableButton);
		builder.append(checkEscape);

		this.controlPanel = builder.getPanel();
	}
}
