/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.check;

import java.awt.*;
import java.awt.event.*;
import java.lang.ref.*;
import java.util.LinkedList;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jvnet.lafwidget.LafWidgetUtilities;
import org.jvnet.lafwidget.utils.FadeConfigurationManager;
import org.jvnet.lafwidget.utils.FadeTracker.FadeKind;
import org.jvnet.lafwidget.utils.LafConstants.TabOverviewKind;
import org.jvnet.substance.*;
import org.jvnet.substance.painter.AlphaControlBackgroundComposite;
import org.jvnet.substance.theme.SubstanceTheme;
import org.jvnet.substance.title.TitleButtonCallback;
import org.jvnet.substance.title.TitleButtonInfo;
import org.jvnet.substance.utils.SubstanceCoreUtilities;
import org.jvnet.substance.utils.SubstanceSizeUtils;
import org.jvnet.substance.utils.SubstanceConstants.MenuGutterFillKind;

import test.Check;
import test.Check.MyMainTabPreviewPainter;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

public class ControlPanelFactory {
	private static SimpleDialog simpleDialog;

	public static JPanel getMainControlPanel(final JFrame mainFrame,
			final JTabbedPane mainTabbedPane,
			final MyMainTabPreviewPainter mainTabPreviewPainter,
			final JToolBar toolbar) {
		FormLayout lm = new FormLayout("right:pref, 4dlu, fill:pref:grow", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
		// builder.setDefaultDialogBorder();

		builder.appendSeparator("Title pane settings");
		final JCheckBox markAsModified = new JCheckBox("Marked modified");
		markAsModified.setSelected(false);
		markAsModified.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				mainFrame.getRootPane().putClientProperty(
						SubstanceLookAndFeel.WINDOW_MODIFIED,
						(markAsModified.isSelected() ? Boolean.TRUE : false));
			}
		});
		builder.append("Modified", markAsModified);

		final JCheckBox heapPanel = new JCheckBox("Has heap panel");
		heapPanel.setSelected(true);
		heapPanel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (heapPanel.isSelected())
					SubstanceLookAndFeel
							.permanentlyShowHeapStatusPanel(mainFrame
									.getRootPane());
				else
					SubstanceLookAndFeel
							.permanentlyHideHeapStatusPanel(mainFrame
									.getRootPane());
			}
		});
		builder.append("Heap panel", heapPanel);

		final JCheckBox customTitlePaneButtons = new JCheckBox("Has custom");
		customTitlePaneButtons.setSelected(false);
		customTitlePaneButtons.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (customTitlePaneButtons.isSelected()) {
					TitleButtonInfo tbInfo1 = new TitleButtonInfo();
					tbInfo1.setActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							JOptionPane.showMessageDialog(mainFrame,
									"Custom button 1 activated");
						}
					});
					tbInfo1.setTooltipText("Tooltip for custom button 1");
					tbInfo1.setButtonCallback(new TitleButtonCallback() {
						public Icon getTitleButtonIcon(
								SubstanceTheme currSubstanceTheme,
								int iconMaxWidth, int iconMaxHeight) {
							return new ImageIcon(
									SubstanceImageCreator
											.overlayEcho(
													SubstanceImageCreator
															.getArrow(
																	SubstanceSizeUtils
																			.getArrowIconWidth(),
																	SubstanceSizeUtils
																			.getArrowIconHeight(),
																	SubstanceSizeUtils
																			.getArrowStrokeWidth(),
																	SwingConstants.WEST,
																	currSubstanceTheme
																			.getActiveTitlePaneTheme()),
													1, 1));
						}
					});

					TitleButtonInfo tbInfo2 = new TitleButtonInfo();
					tbInfo2.setActionListener(new ActionListener() {
						public void actionPerformed(ActionEvent e) {
							JOptionPane.showMessageDialog(mainFrame,
									"Custom button 2 activated");
						}
					});
					tbInfo2.setTooltipText("Tooltip for custom button 2");
					tbInfo2.setButtonCallback(new TitleButtonCallback() {
						public Icon getTitleButtonIcon(
								SubstanceTheme currSubstanceTheme,
								int iconMaxWidth, int iconMaxHeight) {
							return new ImageIcon(
									SubstanceImageCreator
											.overlayEcho(
													SubstanceImageCreator
															.getArrow(
																	SubstanceSizeUtils
																			.getArrowIconWidth(),
																	SubstanceSizeUtils
																			.getArrowIconHeight(),
																	SubstanceSizeUtils
																			.getArrowStrokeWidth(),
																	SwingConstants.EAST,
																	currSubstanceTheme
																			.getActiveTitlePaneTheme()),
													1, 1));
						}
					});

					java.util.List<TitleButtonInfo> tbInfoList = new LinkedList<TitleButtonInfo>();
					tbInfoList.add(tbInfo1);
					tbInfoList.add(tbInfo2);
					SubstanceLookAndFeel.setRootPaneCustomTitleButtons(
							mainFrame.getRootPane(), tbInfoList);
				} else {
					SubstanceLookAndFeel
							.removeRootPaneCustomTitleButtons(mainFrame
									.getRootPane());
				}
			}
		});
		builder.append("Title buttons", customTitlePaneButtons);

		JButton changeTitleButton = new JButton("Change");
		changeTitleButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String random = "abcdefghijklmnopqrstuvwxyz ";
				int length = 60 + (int) (150 * Math.random());
				String title = "";
				while (length > 0) {
					title += random.charAt((int) (random.length() * Math
							.random()));
					length--;
				}
				mainFrame.setTitle(title);
			}
		});
		builder.append("Title string", changeTitleButton);

		builder.appendSeparator("Miscellaneous");

		final JCheckBox useThemedDefaultIconsCheckBox = new JCheckBox(
				"use themed icons");
		useThemedDefaultIconsCheckBox.setSelected(SubstanceCoreUtilities
				.useThemedDefaultIcon(null));
		useThemedDefaultIconsCheckBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						UIManager
								.put(
										SubstanceLookAndFeel.USE_THEMED_DEFAULT_ICONS,
										useThemedDefaultIconsCheckBox
												.isSelected() ? Boolean.TRUE
												: null);
						mainFrame.repaint();
					}
				});
			}
		});
		builder.append("Themed icons", useThemedDefaultIconsCheckBox);

		final JComboBox placementCombo = new JComboBox(new Object[] { "top",
				"bottom", "left", "right" });
		placementCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String selected = (String) placementCombo.getSelectedItem();
				if ("top".equals(selected))
					mainTabbedPane.setTabPlacement(JTabbedPane.TOP);
				if ("bottom".equals(selected))
					mainTabbedPane.setTabPlacement(JTabbedPane.BOTTOM);
				if ("left".equals(selected))
					mainTabbedPane.setTabPlacement(JTabbedPane.LEFT);
				if ("right".equals(selected))
					mainTabbedPane.setTabPlacement(JTabbedPane.RIGHT);
			}
		});
		builder.append("Placement", placementCombo);

		try {
			final JComboBox overviewKindCombo = new JComboBox(new Object[] {
					TabOverviewKind.GRID, TabOverviewKind.MENU_CAROUSEL,
					TabOverviewKind.ROUND_CAROUSEL });
			overviewKindCombo.setSelectedItem(LafWidgetUtilities
					.getTabPreviewPainter(mainTabbedPane).getOverviewKind(
							mainTabbedPane));
			overviewKindCombo.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					mainTabPreviewPainter
							.setTabOverviewKind((TabOverviewKind) overviewKindCombo
									.getSelectedItem());
				}
			});
			overviewKindCombo
					.setRenderer(new SubstanceDefaultListCellRenderer() {
						@Override
						public Component getListCellRendererComponent(
								JList list, Object value, int index,
								boolean isSelected, boolean cellHasFocus) {
							TabOverviewKind kind = (TabOverviewKind) value;
							return super.getListCellRendererComponent(list,
									kind.getName(), index, isSelected,
									cellHasFocus);
						}
					});
			builder.append("Overview kind", overviewKindCombo);
		} catch (NoClassDefFoundError ncdfe) {
		}

		builder.append("Composite",
				new BackgroundCompositeCombo(mainTabbedPane));

		final JComboBox menuGutterFillCombo = new JComboBox(new Object[] {
				MenuGutterFillKind.NONE, MenuGutterFillKind.SOFT,
				MenuGutterFillKind.HARD, MenuGutterFillKind.SOFT_FILL,
				MenuGutterFillKind.HARD_FILL });
		menuGutterFillCombo.setRenderer(new SubstanceDefaultListCellRenderer() {
			@Override
			public Component getListCellRendererComponent(JList list,
					Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				MenuGutterFillKind mgfk = (MenuGutterFillKind) value;
				return super.getListCellRendererComponent(list, mgfk.name()
						.toLowerCase(), index, isSelected, cellHasFocus);
			}
		});
		menuGutterFillCombo.setSelectedItem(SubstanceCoreUtilities
				.getMenuGutterFillKind());
		menuGutterFillCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				UIManager.put(SubstanceLookAndFeel.MENU_GUTTER_FILL_KIND,
						menuGutterFillCombo.getSelectedItem());
			}
		});
		builder.append("Menu fill", menuGutterFillCombo);

		final JCheckBox menuSearch = new JCheckBox("Has menu search");
		menuSearch.setSelected(true);
		menuSearch.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (menuSearch.isSelected())
					SubstanceLookAndFeel.showMenuSearchPanels();
				else
					SubstanceLookAndFeel.hideMenuSearchPanels();
			}
		});
		builder.append("Menu search", menuSearch);

		JButton buttonGlassPane = new JButton("Show");
		buttonGlassPane.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				final JPanel glassPane = new JPanel() {
					@Override
					public void paintComponent(Graphics g) {
						super.paintComponent(g);
						Graphics2D graphics = (Graphics2D) g;
						int height = getHeight();
						int width = getWidth();
						Composite c = AlphaComposite.getInstance(
								AlphaComposite.SRC_OVER, (float) 0.4);
						graphics.setComposite(c);
						for (int i = 0; i < height; i++) {
							Color color = (i % 2 == 0) ? new Color(200, 200,
									255) : new Color(230, 230, 255);
							graphics.setColor(color);
							graphics.drawLine(0, i, width, i);
						}
						Composite c2 = AlphaComposite.getInstance(
								AlphaComposite.SRC_OVER, (float) 1.0);
						graphics.setComposite(c2);
					}
				};
				glassPane.setOpaque(false);
				glassPane.addMouseListener(new MouseAdapter() {
				});
				glassPane.addKeyListener(new KeyAdapter() {
				});
				mainFrame.setGlassPane(glassPane);
				new Thread() {
					public void run() {
						glassPane.setVisible(true);
						try {
							Thread.sleep(5000);
						} catch (InterruptedException ie) {
							ie.printStackTrace();
						}
						glassPane.setVisible(false);
					}
				}.start();
			}
		});
		builder.append("Glass pane", buttonGlassPane);

		builder.appendSeparator("Custom animations");
		final JCheckBox allowFocusLoopAnimations = new JCheckBox(
				"Allow animation");
		allowFocusLoopAnimations.setSelected(FadeConfigurationManager
				.getInstance().fadeAllowed(FadeKind.FOCUS_LOOP_ANIMATION,
						null));
		allowFocusLoopAnimations.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (allowFocusLoopAnimations.isSelected()) {
					FadeConfigurationManager.getInstance().allowFades(
							FadeKind.FOCUS_LOOP_ANIMATION);
				} else {
					FadeConfigurationManager.getInstance().disallowFades(
							FadeKind.FOCUS_LOOP_ANIMATION);
				}
			}
		});
		builder.append("Focus loop", allowFocusLoopAnimations);

		final JCheckBox allowGhostIconAnimations = new JCheckBox(
				"Allow animation");
		allowGhostIconAnimations.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (allowGhostIconAnimations.isSelected()) {
					FadeConfigurationManager.getInstance().allowFades(
							FadeKind.GHOSTING_ICON_ROLLOVER);
				} else {
					FadeConfigurationManager.getInstance().disallowFades(
							FadeKind.GHOSTING_ICON_ROLLOVER);
				}
			}
		});
		builder.append("Icon rollover", allowGhostIconAnimations);

		final JCheckBox allowGhostPressAnimations = new JCheckBox(
				"Allow animation");
		allowGhostPressAnimations.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (allowGhostPressAnimations.isSelected()) {
					FadeConfigurationManager.getInstance().allowFades(
							FadeKind.GHOSTING_BUTTON_PRESS);
				} else {
					FadeConfigurationManager.getInstance().disallowFades(
							FadeKind.GHOSTING_BUTTON_PRESS);
				}
			}
		});
		builder.append("Button press", allowGhostPressAnimations);

		builder.appendSeparator("Toolbar");
		final JCheckBox isToolbarFlat = new JCheckBox("Is flat");
		final JSlider translucencySlider = new JSlider(0, 100, 100);
		translucencySlider.setPreferredSize(new Dimension(50, 24));

		isToolbarFlat.setSelected(true);
		translucencySlider.setEnabled(false);
		isToolbarFlat.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				toolbar.putClientProperty(
						SubstanceLookAndFeel.TOOLBAR_BUTTON_FLAT, Boolean
								.valueOf(isToolbarFlat.isSelected()));
				if (isToolbarFlat.isSelected()) {
					translucencySlider.setValue(100);
					translucencySlider.setEnabled(false);
				} else {
					translucencySlider.setEnabled(true);
				}
				toolbar.repaint();
			}
		});
		builder.append("Flat", isToolbarFlat);

		translucencySlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				toolbar.putClientProperty(
						SubstanceLookAndFeel.BACKGROUND_COMPOSITE,
						new AlphaControlBackgroundComposite(translucencySlider
								.getValue() / 100.f));
				toolbar.repaint();
			}
		});
		builder.append("Opacity", translucencySlider);

		return builder.getPanel();
	}

	public static JPanel getDialogControlPanel(final JFrame mainFrame) {
		FormLayout lm = new FormLayout("right:pref, 4dlu, fill:pref:grow", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);

		builder.appendSeparator("Core choosers");
		JButton bf = new JButton("Open", Check.getIcon("JFileChooserColor16"));
		bf.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser jfc = new JFileChooser();
				jfc.showOpenDialog(mainFrame);
			}
		});
		builder.append("File chooser", bf);

		JButton bc = new JButton("Open", Check.getIcon("JColorChooserColor16"));
		bc.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Color color = JColorChooser.showDialog(mainFrame,
						"Color chooser", new Color(23, 45, 200));
				if (color != null) {
					Check.out("Chosen " + color.toString());
				}
			}
		});
		builder.append("Color chooser", bc);

		builder.appendSeparator("Option panes");

		JButton bopi = new JButton("Show", SubstanceImageCreator
				.getInfoMarkerIcon(16, SubstanceCoreUtilities
						.getOptionPaneColorScheme(
								JOptionPane.INFORMATION_MESSAGE,
								SubstanceLookAndFeel.getActiveColorScheme())));
		bopi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(mainFrame, "Sample info message",
						"Sample title", JOptionPane.INFORMATION_MESSAGE);
			}
		});
		builder.append("Info", bopi);

		JButton bope = new JButton("Show", SubstanceImageCreator
				.getErrorMarkerIcon(16, SubstanceCoreUtilities
						.getOptionPaneColorScheme(JOptionPane.ERROR_MESSAGE,
								SubstanceLookAndFeel.getActiveColorScheme())));
		bope.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(mainFrame,
						"Sample error message", "Sample title",
						JOptionPane.ERROR_MESSAGE);
			}
		});
		builder.append("Error", bope);

		JButton bopw = new JButton("Show", SubstanceImageCreator
				.getWarningMarkerIcon(16, SubstanceCoreUtilities
						.getOptionPaneColorScheme(JOptionPane.WARNING_MESSAGE,
								SubstanceLookAndFeel.getActiveColorScheme())));
		bopw.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(mainFrame,
						"Sample warning message", "Sample title",
						JOptionPane.WARNING_MESSAGE);
			}
		});
		builder.append("Warning", bopw);

		JButton bopq = new JButton("Show", SubstanceImageCreator
				.getQuestionMarkerIcon(16, SubstanceCoreUtilities
						.getOptionPaneColorScheme(JOptionPane.QUESTION_MESSAGE,
								SubstanceLookAndFeel.getActiveColorScheme())));
		bopq.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(mainFrame,
						"Sample question message", "Sample title",
						JOptionPane.QUESTION_MESSAGE);
			}
		});
		builder.append("Question", bopq);

		JButton bopii = new JButton("Show");
		bopii.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						JDialog dialog = new JDialog(mainFrame,
								"Sample dialog", true);
						dialog.setSize(400, 300);
						dialog.setLocationRelativeTo(mainFrame);
						dialog.setLayout(new BorderLayout());
						JPanel panel = new JPanel();
						dialog.add(panel, BorderLayout.CENTER);
						// dialog.setVisible(true);
						JOptionPane.showInternalInputDialog(panel,
								"Sample info message", "Sample title",
								JOptionPane.INFORMATION_MESSAGE);
						dialog.dispose();
					}
				});
			}
		});
		builder.append("Internal input", bopii);

		builder.appendSeparator("Default buttons");

		JButton bd = new JButton("Open");
		bd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SimpleDialog sd = new SimpleDialog();
				sd.setModal(false);
				sd.pack();
				Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
				// center the frame in the physical screen
				sd.setLocation((d.width - sd.getWidth()) / 2, (d.height - sd
						.getHeight()) / 2);
				sd.setVisible(true);
				simpleDialog = sd;
			}
		});
		builder.append("Open a dialog", bd);

		JButton bcd = new JButton("Close");
		bcd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (simpleDialog != null) {
					simpleDialog.removeAll();
					simpleDialog.dispose();
					ReferenceQueue<JButton> weakQueue = new ReferenceQueue<JButton>();
					WeakReference<JButton> weakRef = new WeakReference<JButton>(
							simpleDialog.b1, weakQueue);
					weakRef.enqueue();
					simpleDialog.b1 = null;
					simpleDialog = null;
					System.gc();
					// Wait until the weak reference is on the queue and remove
					// it
					Check.out("Waiting to remove");
					try {
						Reference ref = weakQueue.remove();
						ref.clear();
					} catch (InterruptedException ie) {
						ie.printStackTrace();
						return;
					}
					Check.out("Removed");
				}
			}
		});
		builder.append("Close the dialog", bcd);

		builder.appendSeparator("Miscellaneous");

		JButton btf = new JButton("Show");
		btf.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFrame testFrame = new JFrame("test1");
				testFrame.setSize(262, 100);
				testFrame.setLocation(400, 400);
				testFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				testFrame.setVisible(true);
			}
		});
		builder.append("Regular frame", btf);

		JButton btfU = new JButton("Show");
		btfU.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFrame.setDefaultLookAndFeelDecorated(false);
				JDialog.setDefaultLookAndFeelDecorated(false);

				JFrame testFrame = new JFrame("test undecorated");
				// testFrame.setUndecorated(true);

				testFrame.setSize(262, 100);
				testFrame.setLocation(400, 400);
				testFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				testFrame.setVisible(true);

				JFrame.setDefaultLookAndFeelDecorated(true);
				JDialog.setDefaultLookAndFeelDecorated(true);
			}
		});
		builder.append("Undecorated frame", btfU);

		JButton bcp = new JButton("Open");
		bcp.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {

				JFrame colorFrame = new JFrame();
				final ColorPanel cp1 = new ColorPanel("default");
				final ColorPanel cp2 = new ColorPanel("green");
				cp2.setBackground(Color.green);
				final ColorPanel cp3 = new ColorPanel("red");
				cp3.setBackground(Color.red);
				final ColorPanel cp4 = new ColorPanel("black");
				cp4.setBackground(Color.black);
				// cp2.setBackground(Color.GREEN);
				colorFrame.setLayout(new LayoutManager() {
					public void addLayoutComponent(String name, Component comp) {
					}

					public void layoutContainer(Container parent) {
						int h = parent.getHeight() / 2;
						int w = parent.getWidth() / 2;
						cp1.setBounds(0, 0, w, h);
						cp2.setBounds(0, h, w, parent.getHeight() - h);
						cp3.setBounds(w, 0, w, h + 1);
						cp4.setBounds(w, h + 1, w, parent.getHeight() - h);
					}

					public Dimension minimumLayoutSize(Container parent) {
						return preferredLayoutSize(parent);
					}

					public Dimension preferredLayoutSize(Container parent) {
						return new Dimension(100, 100);
					}

					public void removeLayoutComponent(Component comp) {
					}

				});
				colorFrame.add(cp1);
				colorFrame.add(cp2);
				colorFrame.add(cp3);
				colorFrame.add(cp4);

				colorFrame.setSize(400, 399);
				colorFrame.setLocation(300, 300);
				colorFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				colorFrame.setVisible(true);
			}
		});
		builder.append("Color panels", bcp);

		JButton paneDialog = new JButton("Open");
		paneDialog.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JDialog dialog = new JDialog(mainFrame, true);
				dialog.setTitle("Test text pane in scroll pane");
				JTextPane textPane = new JTextPane();
				String contents = "";
				for (int i = 0; i < 100; i++)
					contents += "This is sample line " + i
							+ " and a lot of other irrelevant text\n";
				textPane.replaceSelection(contents);

				JScrollPane scroll = new JScrollPane(textPane);
				JPanel panel = new JPanel();
				panel.setLayout(new BorderLayout());
				panel.add(scroll, BorderLayout.CENTER);

				dialog.setLayout(new BorderLayout());
				dialog.add(panel, BorderLayout.CENTER);
				dialog.setSize(400, 400);
				dialog.setLocation(400, 200);
				dialog.setVisible(true);
			}
		});
		builder.append("Text pane dialog", paneDialog);

		return builder.getPanel();
	}
}
