/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.check;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.jvnet.lafwidget.LafWidget;
import org.jvnet.lafwidget.list.MutableListModel;
import org.jvnet.lafwidget.utils.FadeTracker;
import org.jvnet.lafwidget.utils.FadeTracker.FadeKind;
import org.jvnet.lafwidget.utils.LafConstants.AnimationKind;
import org.jvnet.substance.SubstanceDefaultListCellRenderer;
import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.combo.WidestComboPopupPrototype;
import org.jvnet.substance.utils.SubstanceColorUtilities;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

public class ListPanel extends ControllablePanel {

	private static class MoveableListModel extends AbstractListModel implements
			MutableListModel {
		protected List<String> model;

		public MoveableListModel() {
			super();
			this.model = new ArrayList<String>();
			for (int i = 0; i < 300; i++) {
				this.model.add("element " + i);
			}
		}

		public Object getElementAt(int index) {
			return this.model.get(index);
		}

		public int getSize() {
			return this.model.size();
		}

		public void moveUp(int index) {
			String entry = this.model.get(index);
			this.model.set(index, this.model.get(index - 1));
			this.model.set(index - 1, entry);
			fireContentsChanged(this, index - 1, index);
		}

		public void moveDown(int index) {
			String entry = this.model.get(index);
			this.model.set(index, this.model.get(index + 1));
			this.model.set(index + 1, entry);
			fireContentsChanged(this, index, index + 1);
		}

		public void delete(int index) {
			this.model.remove(index);
			fireIntervalRemoved(this, index, index);
		}

		public void move(int fromIndex, int toIndex) {
			String from = this.model.remove(fromIndex);
			this.model.add(toIndex, from);
			fireContentsChanged(this, fromIndex, toIndex);
		}
	}

	// private static class SingleListPanel extends ControllablePanel {
	private JList list;

	private JButton bUp;

	private JButton bDown;

	private JButton bDelete;

	private Color oldBackColor;

	private ListCellRenderer oldListCellRenderer;

	public ListPanel() {
		super();
		this.setLayout(new BorderLayout());
		this.list = new JList(new MoveableListModel());

		final JLabel instructional = new JLabel(
				"Drag row to see the smooth drag effect");
		this.add(instructional, BorderLayout.NORTH);
		// create a looping animation to change the label foreground
		// from black to blue and back to draw some attention.
		FadeKind custom = new FadeKind("substance.testapp.list.instructional");
		FadeTracker.getInstance().trackFadeLooping(custom,
				AnimationKind.DEBUG_FAST, instructional, 0, false,
				new FadeTracker.FadeTrackerCallback() {
					public void fadeEnded(FadeKind fadeKind) {
					}

					public void fadePerformed(FadeKind fadeKind,
							float fadeCycle10) {
						instructional.setForeground(SubstanceColorUtilities
								.getInterpolatedColor(Color.black, Color.blue,
										fadeCycle10 / 10.0));
					}
				}, true);

		final JScrollPane jsp = new JScrollPane(this.list);
		this.add(jsp, BorderLayout.CENTER);

		FormLayout lm = new FormLayout("right:pref, 4dlu, fill:pref:grow", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
		builder.appendSeparator("General");

		final JCheckBox watermarkBleed = new JCheckBox("watermark bleed");
		watermarkBleed.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				list.putClientProperty(SubstanceLookAndFeel.WATERMARK_TO_BLEED,
						Boolean.valueOf(watermarkBleed.isSelected()));
				jsp.putClientProperty(SubstanceLookAndFeel.WATERMARK_TO_BLEED,
						Boolean.valueOf(watermarkBleed.isSelected()));
				list.repaint();
			}
		});
		builder.append("Watermark", watermarkBleed);
		final JComboBox animCombo = new JComboBox(new Object[] {
				AnimationKind.NONE, AnimationKind.SLOW, AnimationKind.REGULAR,
				AnimationKind.FAST });
		animCombo.setSelectedItem(AnimationKind.REGULAR);
		animCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				list.putClientProperty(LafWidget.ANIMATION_KIND, animCombo
						.getSelectedItem());
			}
		});
		animCombo.setRenderer(new SubstanceDefaultListCellRenderer() {
			@Override
			public Component getListCellRendererComponent(JList list,
					Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				AnimationKind ak = (AnimationKind) value;
				return super.getListCellRendererComponent(list, ak.getName(),
						index, isSelected, cellHasFocus);
			}
		});
		builder.append("Animation", animCombo);

		this.bUp = new JButton("Move selected up");
		this.bUp.putClientProperty(SubstanceLookAndFeel.FLAT_PROPERTY,
				Boolean.TRUE);

		this.bDown = new JButton("Move selected down");
		this.bDown.putClientProperty(SubstanceLookAndFeel.FLAT_PROPERTY,
				Boolean.TRUE);

		this.bDelete = new JButton("Delete selected");
		this.bDelete.putClientProperty(SubstanceLookAndFeel.FLAT_PROPERTY,
				Boolean.TRUE);

		builder.append("Actions", this.bUp);
		builder.append("", this.bDown);
		builder.append("", this.bDelete);

		this.bUp.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int si = list.getSelectedIndex();
				MoveableListModel mlm = (MoveableListModel) list.getModel();
				mlm.moveUp(si);
				list.setSelectedIndex(si - 1);
			}
		});

		this.bDown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int si = list.getSelectedIndex();
				MoveableListModel mlm = (MoveableListModel) list.getModel();
				mlm.moveDown(si);
				list.setSelectedIndex(si + 1);
			}
		});

		this.bDelete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				MoveableListModel mlm = (MoveableListModel) list.getModel();
				for (int i=list.getMaxSelectionIndex(); i>= list.getMinSelectionIndex(); i--) {
					if (list.isSelectedIndex(i)) {
						mlm.delete(i);
					}
				}
				list.clearSelection();
			}
		});

		this.synchronize();

		this.list.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						SwingUtilities.invokeLater(new Runnable() {
							public void run() {
								synchronize();
							}
						});
					}
				});

		final JComboBox selectionModelCb = new JComboBox(new Object[] {
				"single", "single interval", "multiple interval" });
		selectionModelCb.setSelectedIndex(0);
		this.list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		selectionModelCb.putClientProperty(
				SubstanceLookAndFeel.COMBO_POPUP_PROTOTYPE,
				new WidestComboPopupPrototype());
		selectionModelCb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						String selected = (String) selectionModelCb
								.getSelectedItem();
						if ("single".equals(selected))
							list
									.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
						if ("single interval".equals(selected))
							list
									.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
						if ("multiple interval".equals(selected))
							list
									.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
					}
				});
			}
		});

		builder.append("Selection", selectionModelCb);

		final JCheckBox customBackgroundCb = new JCheckBox(
				"Has pink background");
		customBackgroundCb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (customBackgroundCb.isSelected()) {
					oldBackColor = list.getBackground();
					list.setBackground(new Color(255, 128, 128));
				} else {
					list.setBackground(oldBackColor);
				}
			}
		});

		builder.append("Background", customBackgroundCb);

		final JCheckBox customRenderer = new JCheckBox("Has custom renderer");
		customRenderer.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (customRenderer.isSelected()) {
					oldListCellRenderer = list.getCellRenderer();
					list.setCellRenderer(new MyListCellRenderer());
				} else {
					list.setCellRenderer(oldListCellRenderer);
				}
			}
		});

		builder.append("Renderer", customRenderer);

		this.controlPanel = builder.getPanel();
	}

	public void synchronize() {
		int sc = this.list.getSelectedIndices().length;
		if (sc != 1) {
			this.bUp.setEnabled(false);
			this.bDown.setEnabled(false);
			return;
		}
		this.bDelete.setEnabled(sc > 0);

		int si = this.list.getSelectedIndex();
		this.bUp.setEnabled(si > 0);
		this.bDown.setEnabled(si < (this.list.getModel().getSize() - 1));
//		this.bDelete.setEnabled(true);
	}
}
//
// public ListPanel() {
// this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
//
// SingleListPanel slp1 = new SingleListPanel();
// slp1.list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
// this.add(slp1);
//
// SingleListPanel slp2 = new SingleListPanel();
// slp2.list
// .setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
// slp2.list.setBackground(new Color(255, 128, 128));
// slp2.list.setForeground(new Color(0, 0, 128));
// this.add(slp2);
//
// SingleListPanel slp3 = new SingleListPanel();
// slp3.list
// .setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
// slp3.list.setCellRenderer(new MyListCellRenderer());
// this.add(slp3);
// }
// }
