package test.check;

import java.awt.Event;
import java.awt.Frame;
import java.awt.event.*;
import java.util.LinkedList;
import java.util.Map;

import javax.swing.*;

import org.jvnet.substance.SubstanceImageCreator;
import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.skin.SkinInfo;
import org.jvnet.substance.skin.SubstanceSkin;
import org.jvnet.substance.theme.*;
import org.jvnet.substance.theme.SubstanceTheme.ThemeKind;
import org.jvnet.substance.utils.menu.TraitMenuHandler;

import test.Check;

public class SampleMenuFactory {
	public static JMenu getThemeMenu() {
		Map<String, ThemeInfo> allThemes = SubstanceLookAndFeel.getAllThemes();

		final TraitMenuHandler themeMenuHandler = new TraitMenuHandler();
		JMenu themeMenu = new JMenu("Theme");
		JMenu brightThemes = new JMenu(SubstanceLookAndFeel.getLabelBundle()
				.getString("SystemMenu.themesBright"));
		JMenu coldThemes = new JMenu(SubstanceLookAndFeel.getLabelBundle()
				.getString("SystemMenu.themesCold"));
		JMenu darkThemes = new JMenu(SubstanceLookAndFeel.getLabelBundle()
				.getString("SystemMenu.themesDark"));
		JMenu invertedThemes = SubstanceLookAndFeel.toEnableInvertedThemes() ? new JMenu(
				SubstanceLookAndFeel.getLabelBundle().getString(
						"SystemMenu.themesInverted"))
				: null;
		JMenu negatedThemes = SubstanceLookAndFeel.toEnableNegatedThemes() ? new JMenu(
				SubstanceLookAndFeel.getLabelBundle().getString(
						"SystemMenu.themesNegated"))
				: null;
		JMenu mixedThemes = SubstanceLookAndFeel.hasMixedThemes() ? new JMenu(
				SubstanceLookAndFeel.getLabelBundle().getString(
						"SystemMenu.themesMixed")) : null;
		JMenu jmShiftTheme = new JMenu("Shift themes");

		themeMenu.add(brightThemes);
		themeMenu.add(coldThemes);
		themeMenu.add(darkThemes);
		if (invertedThemes != null)
			themeMenu.add(invertedThemes);
		if (negatedThemes != null)
			themeMenu.add(negatedThemes);
		if (mixedThemes != null)
			themeMenu.add(mixedThemes);

		for (Map.Entry<String, ThemeInfo> themeEntry : allThemes.entrySet()) {
			final ThemeInfo themeInfo = themeEntry.getValue();
			final String themeClassName = themeInfo.getClassName();
			JRadioButtonMenuItem jmiTheme = new JRadioButtonMenuItem(themeEntry
					.getKey());
			themeMenuHandler.addTraitButton(themeInfo.getDisplayName(),
					jmiTheme);
			jmiTheme.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					setTheme(themeInfo);
				}
			});
			try {
				switch (themeInfo.getThemeKind()) {
				case MIXED:
					MixedThemeInfo mixedThemeInfo = (MixedThemeInfo) themeInfo;
					String[] themeClassNames = mixedThemeInfo
							.getThemeClassNames();
					SubstanceTheme[] themeInstances = new SubstanceTheme[themeClassNames.length];
					for (int i = 0; i < themeClassNames.length; i++) {
						Class themeClass = Class.forName(themeClassNames[i]);
						themeInstances[i] = (SubstanceTheme) themeClass
								.newInstance();
					}

					SubstanceTheme mixTheme = new SubstanceMixTheme(
							themeInstances);
					jmiTheme.setIcon(SubstanceImageCreator
							.getThemeIcon(mixTheme));
					break;
				default:
					Class themeClass = Class.forName(themeClassName);
					SubstanceTheme theme = (SubstanceTheme) themeClass
							.newInstance();
					if (themeInfo.getThemeKind() == ThemeKind.INVERTED)
						theme = new SubstanceInvertedTheme(theme);
					if (themeInfo.getThemeKind() == ThemeKind.NEGATED)
						theme = new SubstanceNegatedTheme(theme);
					jmiTheme.setIcon(SubstanceImageCreator.getThemeIcon(theme));
				}
			} catch (Exception exc) {
				continue;
			}
			if (themeEntry.getKey().equals(
					SubstanceLookAndFeel.getCurrentThemeName())) {
				jmiTheme.setSelected(true);
			}
			switch (themeEntry.getValue().getThemeKind()) {
			case BRIGHT:
				brightThemes.add(jmiTheme);
				break;
			case COLD:
				coldThemes.add(jmiTheme);
				break;
			case DARK:
				darkThemes.add(jmiTheme);
				break;
			case INVERTED:
				invertedThemes.add(jmiTheme);
				break;
			case NEGATED:
				negatedThemes.add(jmiTheme);
				break;
			case MIXED:
				mixedThemes.add(jmiTheme);
				break;
			// case DARK_MIXED:
			// mixedDarkThemes.add(jmiTheme);
			// break;
			}
		}

		ThemeChangeListener themeChangeListener = new ThemeChangeListener() {
			public void themeChanged() {
				themeMenuHandler.selectTraitButton(SubstanceLookAndFeel
						.getTheme());
			}
		};
		SubstanceLookAndFeel.registerThemeChangeListener(themeChangeListener);

		JMenuItem jmiAquaTint10Theme = new JMenuItem("Tint Aqua 10%");
		jmiAquaTint10Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().tint(0.1));
			}
		});
		jmShiftTheme.add(jmiAquaTint10Theme);

		JMenuItem jmiAquaTint25Theme = new JMenuItem("Tint Aqua 25%");
		jmiAquaTint25Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().tint(0.25));
			}
		});
		jmShiftTheme.add(jmiAquaTint25Theme);

		JMenuItem jmiAquaTint40Theme = new JMenuItem("Tint Aqua 40%");
		jmiAquaTint40Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().tint(0.4));
			}
		});
		jmShiftTheme.add(jmiAquaTint40Theme);

		JMenuItem jmiAquaTone10Theme = new JMenuItem("Tone Aqua 10%");
		jmiAquaTone10Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().tone(0.1));
			}
		});
		jmShiftTheme.add(jmiAquaTone10Theme);

		JMenuItem jmiAquaTone25Theme = new JMenuItem("Tone Aqua 25%");
		jmiAquaTone25Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().tone(0.25));
			}
		});
		jmShiftTheme.add(jmiAquaTone25Theme);

		JMenuItem jmiAquaTone40Theme = new JMenuItem("Tone Aqua 40%");
		jmiAquaTone40Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().tone(0.4));
			}
		});
		jmShiftTheme.add(jmiAquaTone40Theme);

		JMenuItem jmiAquaShade10Theme = new JMenuItem("Shade Aqua 10%");
		jmiAquaShade10Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().shade(0.1));
			}
		});
		jmShiftTheme.add(jmiAquaShade10Theme);

		JMenuItem jmiAquaShade25Theme = new JMenuItem("Shade Aqua 25%");
		jmiAquaShade25Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().shade(0.25));
			}
		});
		jmShiftTheme.add(jmiAquaShade25Theme);

		JMenuItem jmiAquaShade40Theme = new JMenuItem("Shade Aqua 40%");
		jmiAquaShade40Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().shade(0.4));
			}
		});
		jmShiftTheme.add(jmiAquaShade40Theme);

		JMenuItem jmiSteelBlueTone25Theme = new JMenuItem("Tone Steel Blue 25%");
		jmiSteelBlueTone25Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceSteelBlueTheme().tone(0.25));
			}
		});
		jmShiftTheme.add(jmiSteelBlueTone25Theme);

		JMenuItem jmiLimeGreenShade25Theme = new JMenuItem(
				"Shade Lime Green 25%");
		jmiLimeGreenShade25Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceLimeGreenTheme().shade(0.25));
			}
		});
		jmShiftTheme.add(jmiLimeGreenShade25Theme);

		JMenuItem jmiBarbyPinkTint25Theme = new JMenuItem("Tint Barby Pink 25%");
		jmiBarbyPinkTint25Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceBarbyPinkTheme().tint(0.25));
			}
		});
		jmShiftTheme.add(jmiBarbyPinkTint25Theme);

		JMenuItem jmiAquaBottleGreenMixTint25Theme = new JMenuItem(
				"Tint Aqua Bottle Green Mix 25%");
		jmiAquaBottleGreenMixTint25Theme
				.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						setTheme(new SubstanceMixTheme(
								new SubstanceAquaTheme(),
								new SubstanceBottleGreenTheme()).tint(0.25));
					}
				});
		jmShiftTheme.add(jmiAquaBottleGreenMixTint25Theme);

		JMenuItem jmiAquaDesaturate50Theme = new JMenuItem(
				"Aqua Desaturate 50%");
		jmiAquaDesaturate50Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().saturate(-0.5));
			}
		});
		jmShiftTheme.add(jmiAquaDesaturate50Theme);

		JMenuItem jmiAquaSaturate50Theme = new JMenuItem("Aqua Saturate 50%");
		jmiAquaSaturate50Theme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceAquaTheme().saturate(0.5));
			}
		});
		jmShiftTheme.add(jmiAquaSaturate50Theme);

		jmShiftTheme.addSeparator();

		JMenuItem jmiGreenBlendTheme = new JMenuItem("Green aqua blend");
		jmiGreenBlendTheme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceMixTheme(new SubstanceBlendBiTheme(
						new SubstanceBottleGreenTheme(),
						new SubstanceLimeGreenTheme(), 0.6),
						new SubstanceBlendBiTheme(new SubstancePurpleTheme(),
								new SubstanceAquaTheme(), 0.4)));
			}
		});
		jmShiftTheme.add(jmiGreenBlendTheme);

		themeMenu.add(jmShiftTheme);

		themeMenu.addSeparator();

		JMenu changeTheme = new JMenu("Change current");

		JMenuItem invertCurrent = new JMenuItem("Inverted current");
		invertCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().invert());
			}
		});
		changeTheme.add(invertCurrent);

		JMenuItem negateCurrent = new JMenuItem("Negated current");
		negateCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().negate());
			}
		});
		changeTheme.add(negateCurrent);

		changeTheme.addSeparator();

		JMenuItem tintCurrent = new JMenuItem("Tinted current");
		tintCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().tint(0.3));
			}
		});
		changeTheme.add(tintCurrent);

		JMenuItem toneCurrent = new JMenuItem("Toned current");
		toneCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().tone(0.3));
			}
		});
		changeTheme.add(toneCurrent);

		JMenuItem shadeCurrent = new JMenuItem("Shaded current");
		shadeCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().shade(0.3));
			}
		});
		changeTheme.add(shadeCurrent);

		changeTheme.addSeparator();

		JMenuItem saturateCurrent = new JMenuItem("Saturated current");
		saturateCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().saturate(0.3));
			}
		});
		changeTheme.add(saturateCurrent);

		JMenuItem saturateActiveCurrent = new JMenuItem(
				"Saturated active current");
		saturateActiveCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().saturate(0.3, false));
			}
		});
		changeTheme.add(saturateActiveCurrent);

		JMenuItem desaturateCurrent = new JMenuItem("Desaturated current");
		desaturateCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().saturate(-0.3));
			}
		});
		changeTheme.add(desaturateCurrent);

		JMenuItem desaturateActiveCurrent = new JMenuItem(
				"Desaturated active current");
		desaturateActiveCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().saturate(-0.3, false));
			}
		});
		changeTheme.add(desaturateActiveCurrent);

		changeTheme.addSeparator();

		JMenuItem hueShiftCurrent_p002 = new JMenuItem("Hue-shift current 0.02");
		hueShiftCurrent_p002.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().hueShift(0.02));
			}
		});
		changeTheme.add(hueShiftCurrent_p002);

		JMenuItem hueShiftCurrent_m002 = new JMenuItem(
				"Hue-shift current -0.02");
		hueShiftCurrent_m002.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().hueShift(-0.02));
			}
		});
		changeTheme.add(hueShiftCurrent_m002);

		JMenuItem hueShiftCurrent_p01 = new JMenuItem("Hue-shift current 0.1");
		hueShiftCurrent_p01.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().hueShift(0.1));
			}
		});
		changeTheme.add(hueShiftCurrent_p01);

		JMenuItem hueShiftCurrent_m01 = new JMenuItem("Hue-shift current -0.1");
		hueShiftCurrent_m01.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().hueShift(-0.1));
			}
		});
		changeTheme.add(hueShiftCurrent_m01);

		JMenuItem hueShiftCurrent_p05 = new JMenuItem("Hue-shift current 0.5");
		hueShiftCurrent_p05.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().hueShift(0.5));
			}
		});
		changeTheme.add(hueShiftCurrent_p05);

		JMenuItem hueShiftCurrent_m05 = new JMenuItem("Hue-shift current -0.5");
		hueShiftCurrent_m05.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().hueShift(-0.5));
			}
		});
		changeTheme.add(hueShiftCurrent_m05);

		changeTheme.addSeparator();

		final JMenuItem restoreOrig = new JMenuItem("Restore original");
		restoreOrig
				.setEnabled(SubstanceLookAndFeel.getTheme() instanceof SubstanceWrapperTheme);
		restoreOrig.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(((SubstanceWrapperTheme) SubstanceLookAndFeel
						.getTheme()).getOriginalTheme());
			}
		});
		SubstanceLookAndFeel
				.registerThemeChangeListener(new ThemeChangeListener() {
					public void themeChanged() {
						restoreOrig
								.setEnabled(SubstanceLookAndFeel.getTheme() instanceof SubstanceWrapperTheme);
					}
				});
		changeTheme.add(restoreOrig);

		themeMenu.add(changeTheme);

		JMenu cbMenu = new JMenu("Color blindness");
		JMenuItem protanopiaCurrent = new JMenuItem("Protanopia current");
		protanopiaCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().protanopia());
			}
		});
		cbMenu.add(protanopiaCurrent);
		JMenuItem deuteranopiaCurrent = new JMenuItem("Deuteranopia current");
		deuteranopiaCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().deuteranopia());
			}
		});
		cbMenu.add(deuteranopiaCurrent);
		JMenuItem tritanopiaCurrent = new JMenuItem("Tritanopia current");
		tritanopiaCurrent.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(SubstanceLookAndFeel.getTheme().tritanopia());
			}
		});
		cbMenu.add(tritanopiaCurrent);
		themeMenu.add(cbMenu);

		themeMenu.addSeparator();

		JMenuItem jmiCaribbeanBlueTheme = new JMenuItem("User - Caribbean blue");
		jmiCaribbeanBlueTheme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceTheme(new CaribbeanBlueColorScheme(),
						"Caribbean blue", SubstanceTheme.ThemeKind.BRIGHT));
			}
		});
		themeMenu.add(jmiCaribbeanBlueTheme);

		JMenuItem jmiCrimsonTheme = new JMenuItem("User - Crimson");
		jmiCrimsonTheme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setTheme(new SubstanceTheme(new CrimsonColorScheme(),
						"Crimson", SubstanceTheme.ThemeKind.BRIGHT));
			}
		});
		themeMenu.add(jmiCrimsonTheme);

		return themeMenu;
	}

	public static void setTheme(final SubstanceTheme theme) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					SubstanceLookAndFeel.setCurrentTheme(theme);
				} catch (Exception exc) {
					exc.printStackTrace();
				}
				for (Frame frame : Frame.getFrames())
					SwingUtilities.updateComponentTreeUI(frame);
			}
		});
	}

	public static void setTheme(final ThemeInfo themeInfo) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					SubstanceLookAndFeel.setCurrentTheme(themeInfo);
				} catch (Exception exc) {
					exc.printStackTrace();
				}
				for (Frame frame : Frame.getFrames())
					SwingUtilities.updateComponentTreeUI(frame);
			}
		});
	}

	public static JMenu getTestMenu() {
		JMenu testMenu = new JMenu("Test");
		testMenu.setMnemonic('1');
		int mcount = 0;
		for (LinkedList<JMenuItem> miList : getTestMenuItems()) {
			if (mcount > 0) {
				if (mcount % 2 == 0)
					testMenu.addSeparator();
				else
					testMenu.add(new JSeparator());
			}
			for (JMenuItem menuItem : miList) {
				testMenu.add(menuItem);
			}
			mcount++;
		}

		return testMenu;
	}

	public static LinkedList<LinkedList<JMenuItem>> getTestMenuItems() {
		LinkedList<LinkedList<JMenuItem>> result = new LinkedList<LinkedList<JMenuItem>>();
		LinkedList<JMenuItem> list1 = new LinkedList<JMenuItem>();
		final JMenuItem jmi1 = new JMenuItem("Menu item enabled", Check
				.getIcon("flag_sweden"));
		jmi1.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A,
				Event.CTRL_MASK));
		JMenuItem jmi2 = new JMenuItem("Menu item disabled");
		jmi2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_B,
				Event.CTRL_MASK | Event.ALT_MASK));
		jmi2.setEnabled(false);

		list1.add(jmi1);
		list1.add(jmi2);
		result.add(list1);

		LinkedList<JMenuItem> list2 = new LinkedList<JMenuItem>();
		final JCheckBoxMenuItem jcbmi1 = new JCheckBoxMenuItem(
				"Check enabled selected", SubstanceImageCreator.getHexaMarker(
						2, SubstanceLookAndFeel.getTheme()));
		jcbmi1.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C,
				Event.CTRL_MASK));
		jcbmi1.setSelected(true);
		JCheckBoxMenuItem jcbmi2 = new JCheckBoxMenuItem(
				"Check enabled unselected");
		jcbmi2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D,
				Event.CTRL_MASK));
		jcbmi2.setSelected(false);
		JCheckBoxMenuItem jcbmi3 = new JCheckBoxMenuItem(
				"Check disabled selected");
		jcbmi3.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,
				Event.CTRL_MASK));
		jcbmi3.setSelected(true);
		jcbmi3.setEnabled(false);
		final JCheckBoxMenuItem jcbmi4 = new JCheckBoxMenuItem(
				"Check disabled unselected", SubstanceImageCreator
						.getHexaMarker(3, SubstanceLookAndFeel.getTheme()));
		jcbmi4.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F,
				Event.CTRL_MASK));
		jcbmi4.setSelected(false);
		jcbmi4.setEnabled(false);

		list2.add(jcbmi1);
		list2.add(jcbmi2);
		list2.add(jcbmi3);
		list2.add(jcbmi4);
		result.add(list2);

		LinkedList<JMenuItem> list3 = new LinkedList<JMenuItem>();
		final JRadioButtonMenuItem jrbmi1 = new JRadioButtonMenuItem(
				"Radio enabled selected", SubstanceImageCreator.getHexaMarker(
						4, SubstanceLookAndFeel.getTheme()));
		jrbmi1.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G,
				Event.CTRL_MASK));
		jrbmi1.setSelected(true);
		JRadioButtonMenuItem jrbmi2 = new JRadioButtonMenuItem(
				"Radio enabled unselected");
		jrbmi2.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_H,
				Event.CTRL_MASK));
		jrbmi2.setSelected(false);
		ButtonGroup bgRadioMenu1 = new ButtonGroup();
		bgRadioMenu1.add(jrbmi1);
		bgRadioMenu1.add(jrbmi2);
		JRadioButtonMenuItem jrbmi3 = new JRadioButtonMenuItem(
				"Radio disabled selected");
		jrbmi3.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I,
				Event.CTRL_MASK));
		jrbmi3.setSelected(true);
		jrbmi3.setEnabled(false);
		JRadioButtonMenuItem jrbmi4 = new JRadioButtonMenuItem(
				"Radio disabled unselected");
		jrbmi4.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_J,
				Event.CTRL_MASK));
		jrbmi4.setSelected(false);
		jrbmi4.setEnabled(false);
		ButtonGroup bgRadioMenu2 = new ButtonGroup();
		bgRadioMenu2.add(jrbmi3);
		bgRadioMenu2.add(jrbmi4);

		list3.add(jrbmi1);
		list3.add(jrbmi2);
		list3.add(jrbmi3);
		list3.add(jrbmi4);
		result.add(list3);

		LinkedList<JMenuItem> list4 = new LinkedList<JMenuItem>();
		JMenu submenu1 = new JMenu("submenu1");
		submenu1.add(new JMenuItem("submenu item1"));
		submenu1.add(new JMenuItem("submenu item2"));
		submenu1.add(new JMenuItem("submenu item3"));
		final JMenu submenu11 = new JMenu("submenu1-1");
		submenu11.setIcon(SubstanceImageCreator.getHexaMarker(5,
				SubstanceLookAndFeel.getTheme()));
		submenu11.add(new JMenuItem("submenu item111"));
		submenu11.add(new JMenuItem("submenu item112"));
		submenu11.add(new JMenuItem("submenu item113"));
		submenu11.add(new JMenuItem("submenu item114"));
		submenu1.add(submenu11);
		final JMenu submenu12 = new JMenu("submenu1-2");
		submenu12.setIcon(SubstanceImageCreator.getHexaMarker(5,
				SubstanceLookAndFeel.getTheme()));
		submenu12.add(new JMenuItem("submenu item121"));
		submenu12.add(new JMenuItem("submenu item122"));
		submenu12.add(new JMenuItem("submenu item123"));
		submenu12.add(new JMenuItem("submenu item124"));
		submenu1.add(submenu12);
		final JMenu submenu13 = new JMenu("submenu1-3");
		submenu13.setIcon(SubstanceImageCreator.getHexaMarker(5,
				SubstanceLookAndFeel.getTheme()));
		submenu13.add(new JMenuItem("submenu item131"));
		submenu13.add(new JMenuItem("submenu item132"));
		submenu13.add(new JMenuItem("submenu item133"));
		submenu13.add(new JMenuItem("submenu item134"));
		submenu13.add(new JMenuItem("submenu item135"));
		submenu1.add(submenu13);
		list4.add(submenu1);

		JMenu submenu2 = new JMenu("submenu2");
		submenu2.add(new JMenuItem("submenu item1"));
		submenu2.add(new JMenuItem("submenu item2"));
		submenu2.add(new JMenuItem("submenu item3"));
		final JMenu submenu21 = new JMenu("submenu2-1");
		submenu21.setIcon(SubstanceImageCreator.getHexaMarker(5,
				SubstanceLookAndFeel.getTheme()));
		submenu21.add(new JMenuItem("submenu item211"));
		submenu21.add(new JMenuItem("submenu item212"));
		submenu21.add(new JMenuItem("submenu item213"));
		submenu21.add(new JMenuItem("submenu item214"));
		submenu2.add(submenu21);
		final JMenu submenu22 = new JMenu("submenu2-2");
		submenu22.setIcon(SubstanceImageCreator.getHexaMarker(5,
				SubstanceLookAndFeel.getTheme()));
		submenu22.add(new JMenuItem("submenu item221"));
		submenu22.add(new JMenuItem("submenu item222"));
		submenu22.add(new JMenuItem("submenu item223"));
		submenu22.add(new JMenuItem("submenu item224"));
		submenu2.add(submenu22);
		final JMenu submenu23 = new JMenu("submenu2-3");
		submenu23.setIcon(SubstanceImageCreator.getHexaMarker(5,
				SubstanceLookAndFeel.getTheme()));
		submenu23.add(new JMenuItem("submenu item231"));
		submenu23.add(new JMenuItem("submenu item232"));
		submenu23.add(new JMenuItem("submenu item233"));
		submenu23.add(new JMenuItem("submenu item234"));
		submenu2.add(submenu23);
		list4.add(submenu2);

		result.add(list4);

		SubstanceLookAndFeel
				.registerThemeChangeListener(new ThemeChangeListener() {
					public void themeChanged() {
						// jmi1.setIcon(SubstanceImageCreator.getHexaMarker(1,
						// SubstanceLookAndFeel.getTheme()));
						jcbmi1.setIcon(SubstanceImageCreator.getHexaMarker(2,
								SubstanceLookAndFeel.getTheme()));
						jcbmi4.setIcon(SubstanceImageCreator.getHexaMarker(3,
								SubstanceLookAndFeel.getTheme()));
						jrbmi1.setIcon(SubstanceImageCreator.getHexaMarker(4,
								SubstanceLookAndFeel.getTheme()));
						submenu11.setIcon(SubstanceImageCreator.getHexaMarker(
								5, SubstanceLookAndFeel.getTheme()));
						submenu12.setIcon(SubstanceImageCreator.getHexaMarker(
								5, SubstanceLookAndFeel.getTheme()));
						submenu13.setIcon(SubstanceImageCreator.getHexaMarker(
								5, SubstanceLookAndFeel.getTheme()));
						submenu21.setIcon(SubstanceImageCreator.getHexaMarker(
								5, SubstanceLookAndFeel.getTheme()));
						submenu22.setIcon(SubstanceImageCreator.getHexaMarker(
								5, SubstanceLookAndFeel.getTheme()));
						submenu23.setIcon(SubstanceImageCreator.getHexaMarker(
								5, SubstanceLookAndFeel.getTheme()));
					}
				});

		return result;
	}

	public static JMenu getSkinMenu() {
		JMenu jmSkin = new JMenu("Skins");
		Map<String, SkinInfo> skinMap = SubstanceLookAndFeel.getAllSkins();
		for (final Map.Entry<String, SkinInfo> entry : skinMap.entrySet()) {
			JMenuItem jmiSkin = new JMenuItem(entry.getValue().getDisplayName());
			jmiSkin.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					SubstanceLookAndFeel.setSkin(entry.getValue()
							.getClassName());
				}
			});

			try {
				SubstanceSkin skin = ((SubstanceSkin) Class.forName(
						entry.getValue().getClassName()).newInstance());
				jmiSkin.setIcon(new ImageIcon(SubstanceImageCreator
						.getThemeImage(new ImageIcon(SampleMenuFactory.class
								.getClassLoader().getResource(
										"test/resource/image-x-generic.png")),
								skin.getTheme(), false)));
			} catch (Exception exc) {
			}

			jmSkin.add(jmiSkin);
		}
		return jmSkin;
	}
}
