/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.check;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.*;

import org.jvnet.substance.SubstanceLookAndFeel;


public class TabNumberedPanel extends NumberedPanel {
	public int number;

	public static LinkedList<TabNumberedPanel> closed = new LinkedList<TabNumberedPanel>();

	public TabNumberedPanel(final JTabbedPane tabContainer, int number) {
		super(number);
		this.setLayout(new BorderLayout());

		JPanel southPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		JButton closeButton = new JButton("Close tab " + number);
		closeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						tabContainer.removeTabAt(tabContainer
								.indexOfComponent(TabNumberedPanel.this));
						closed.add(TabNumberedPanel.this);
					}
				});
			}
		});
		southPanel.add(closeButton);

		final JTextField sizeField = new JTextField(2);
		JButton setSizeButton = new JButton("Set close size to ");
		setSizeButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						TabNumberedPanel.this
								.putClientProperty(
										SubstanceLookAndFeel.TABBED_PANE_CLOSE_BUTTONS_SIZE,
										new Integer(sizeField.getText()));
						tabContainer.repaint();
					}
				});
			}
		});
		southPanel.add(setSizeButton);
		southPanel.add(sizeField);

		JButton markClosedAnimation = new JButton(
				"Set modified animation on close button");
		markClosedAnimation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				TabNumberedPanel.this
						.putClientProperty(
								SubstanceLookAndFeel.TABBED_PANE_CLOSE_BUTTONS_MODIFIED_ANIMATION,
								Boolean.TRUE);
			}
		});
		southPanel.add(markClosedAnimation);

		this.add(southPanel, BorderLayout.SOUTH);
	}
}