/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.check;

import java.awt.*;
import java.awt.event.*;
import java.util.Calendar;
import java.util.Date;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.*;

import org.jvnet.lafwidget.utils.FadeConfigurationManager;
import org.jvnet.lafwidget.utils.FadeTracker;
import org.jvnet.lafwidget.utils.FadeTracker.FadeKind;
import org.jvnet.lafwidget.utils.LafConstants.AnimationKind;
import org.jvnet.substance.*;
import org.jvnet.substance.theme.SubstanceTheme;
import org.jvnet.substance.theme.ThemeInfo;
import org.jvnet.substance.utils.SubstanceColorUtilities;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import test.Check;

public class TablePanel extends ControllablePanel {
	private JTable table;

	private Object[][] data;

	private Class[] columns;
	
	private static class MyColorTableRenderer extends JLabel implements
			TableCellRenderer {
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Color color = (Color) value;
			this.setForeground(color);
			this.setBackground(SubstanceColorUtilities.invertColor(color));
			this.setText("row " + row);
			return this;
		}
	}

	private static class MyFloatTableRenderer extends JLabel implements
			TableCellRenderer {
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			int c = (10 * row) % 255;
			Color color = new Color(c, c, c);
			this.setForeground(new Color(255 - c, 0, 0));
			this.setBackground(color);
			this.setText(value.toString());
			return this;
		}
	}

	public TablePanel() {
		final int rows = 400;
		final int cols = 10;
		this.data = new Object[rows][cols];
		this.columns = new Class[] { String.class, JComboBox.class,
				Boolean.class, Byte.class, Float.class, Double.class,
				String.class, Date.class, ImageIcon.class, Color.class };

		for (int i = 0; i < rows; i++) {
			this.data[i][0] = "cell " + i + ":" + 0;
			this.data[i][1] = "predef";
			this.data[i][2] = new Boolean(i % 2 == 0);
			this.data[i][3] = new Byte((byte) i);
			this.data[i][4] = new Float((float) i);
			this.data[i][5] = new Double((double) i);
			this.data[i][6] = "cell " + i + ":" + 6;
			Calendar cal = Calendar.getInstance();
			cal.set(2000 + i, 1 + i, 1 + i);
			this.data[i][7] = cal.getTime();
			int count = 0;
			if (UIManager.getLookAndFeel() instanceof SubstanceLookAndFeel) {
				for (ThemeInfo themeInfo : SubstanceLookAndFeel.getAllThemes()
						.values()) {
					if (count++ == i) {
						try {
							this.data[i][8] = SubstanceImageCreator
									.getHexaMarker(6, (SubstanceTheme) Class
											.forName(themeInfo.getClassName())
											.newInstance());
						} catch (Exception exc) {
						}
					}
				}
			}

			int comp = i * 20;
			int red = (comp / 3) % 255;
			int green = (comp / 2) % 255;
			int blue = comp % 255;
			this.data[i][9] = new Color(red, green, blue);
		}

		TableModel dataModel = new AbstractTableModel() {
			@Override
			public Class<?> getColumnClass(int columnIndex) {
				return columns[columnIndex];
			}

			public int getColumnCount() {
				return cols;
			}

			public int getRowCount() {
				return rows;
			}

			public Object getValueAt(int row, int col) {
				return data[row][col];
			}

			@Override
			public String getColumnName(int column) {
				return this.getColumnClass(column).getSimpleName();
			}

			@Override
			public boolean isCellEditable(int rowIndex, int columnIndex) {
				return (rowIndex % 2 == 0);
			}

			@Override
			public void setValueAt(Object value, int row, int col) {
				data[row][col] = value;
				fireTableCellUpdated(row, col);
			}
		};
		// TableColumnModel columnModel = new DefaultTableColumnModel() {
		// @Override public int getColumnCount() {
		// return 10;
		// }
		//
		// @Override public TableColumn getColumn(int columnIndex) {
		// return new TableColumn(
		// }
		// };
		this.table = new JTable(dataModel);
		this.table.setDefaultRenderer(Color.class, new MyColorTableRenderer());
		this.table.setDefaultRenderer(Float.class, new MyFloatTableRenderer());
		final JScrollPane tableScrollpane = new JScrollPane(this.table);

		JComboBox combo = new JComboBox(new Object[] { "aa", "bb", "cc" });
		combo.setBorder(null);
		this.table.getColumnModel().getColumn(1).setCellEditor(
				new DefaultCellEditor(combo));

		// JTable table = new JTable(
		// new String[][]{{"cell11", "cell12", "cell13", "cell14"},
		// {"cell21", "cell22", "cell23", "cell24"},
		// {"cell31", "cell32", "cell33", "cell34"},
		// {"cell41", "cell42", "cell43", "cell44"}},
		// new String[]{"column1", "column2", "column3", "column4"});

		this.table
				.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		// ListSelectionModel rowSM = this.table.getSelectionModel();
		// rowSM.addListSelectionListener(new ListSelectionListener() {
		// public void valueChanged(ListSelectionEvent e) {
		// // Ignore extra messages.
		// if (e.getValueIsAdjusting()) {
		// return;
		// }
		//
		// ListSelectionModel lsm = (ListSelectionModel) e.getSource();
		// if (lsm.isSelectionEmpty()) {
		// Check.out("No rows are selected.");
		// } else {
		// int selectedRow = lsm.getMinSelectionIndex();
		// Check.out("Row " + selectedRow + " is now selected.");
		// }
		// }
		// });
		// We allow both row and column selection, which
		// implies that we *really* want to allow individual
		// cell selection.
		this.table.setCellSelectionEnabled(true);
		this.table.setColumnSelectionAllowed(true);
		// ListSelectionModel colSM = this.table.getColumnModel()
		// .getSelectionModel();
		// colSM.addListSelectionListener(new ListSelectionListener() {
		// public void valueChanged(ListSelectionEvent e) {
		// // Ignore extra messages.
		// if (e.getValueIsAdjusting()) {
		// return;
		// }
		//
		// ListSelectionModel lsm = (ListSelectionModel) e.getSource();
		// if (lsm.isSelectionEmpty()) {
		// Check.out("No columns are selected.");
		// } else {
		// int selectedCol = lsm.getMinSelectionIndex();
		// Check.out("Column " + selectedCol + " is now selected.");
		// }
		// }
		// });

		this.table.setShowGrid(true);
		this.table.setDragEnabled(false);
		this.table
				.setTableHeader(new JTableHeader(this.table.getColumnModel()));

		this.setLayout(new BorderLayout());
		this.add(tableScrollpane, BorderLayout.CENTER);

		final JLabel instructional = new JLabel("Every odd row is editable");
		this.add(instructional, BorderLayout.NORTH);
		// create a looping animation to change the label foreground
		// from black to blue and back to draw some attention.
		FadeKind custom = new FadeKind("substance.testapp.table.instructional");
		FadeTracker.getInstance().trackFadeLooping(custom,
				AnimationKind.DEBUG_FAST, instructional, 0, false,
				new FadeTracker.FadeTrackerCallback() {
					public void fadeEnded(FadeKind fadeKind) {
					}

					public void fadePerformed(FadeKind fadeKind,
							float fadeCycle10) {
						instructional.setForeground(SubstanceColorUtilities
								.getInterpolatedColor(Color.black, Color.blue,
										fadeCycle10 / 10.0));
					}
				}, true);

		FormLayout lm = new FormLayout("right:pref, 4dlu, fill:pref:grow", "");
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);

		builder.appendSeparator("Table settings");
		final JCheckBox areRowsSelectable = new JCheckBox("Rows selectable");
		areRowsSelectable.setSelected(table.getRowSelectionAllowed());
		areRowsSelectable.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.setRowSelectionAllowed(areRowsSelectable.isSelected());
			}
		});
		builder.append("Selectable", areRowsSelectable);

		final JCheckBox areColsSelectable = new JCheckBox("Cols selectable");
		areColsSelectable.setSelected(table.getColumnSelectionAllowed());
		areColsSelectable.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.setColumnSelectionAllowed(areColsSelectable.isSelected());
			}
		});
		builder.append("", areColsSelectable);

		final JCheckBox watermarkBleed = new JCheckBox("Watermark bleed");
		watermarkBleed.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.putClientProperty(
						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean
								.valueOf(watermarkBleed.isSelected()));
				tableScrollpane.putClientProperty(
						SubstanceLookAndFeel.WATERMARK_TO_BLEED, Boolean
								.valueOf(watermarkBleed.isSelected()));
				tableScrollpane.repaint();
			}
		});
		builder.append("Watermark", watermarkBleed);

		final JCheckBox linesVertical = new JCheckBox("Vertical visible");
		linesVertical.setSelected(table.getShowVerticalLines());
		linesVertical.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.setShowVerticalLines(linesVertical.isSelected());
			}
		});
		builder.append("Lines", linesVertical);
		final JCheckBox linesHorizontal = new JCheckBox("Horizontal visible");
		linesHorizontal.setSelected(table.getShowHorizontalLines());
		linesHorizontal.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.setShowHorizontalLines(linesHorizontal.isSelected());
			}
		});
		builder.append("", linesHorizontal);

		final JComboBox resizeModeCombo = new JComboBox(
				new Object[] { JTable.AUTO_RESIZE_OFF,
						JTable.AUTO_RESIZE_NEXT_COLUMN,
						JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS,
						JTable.AUTO_RESIZE_LAST_COLUMN,
						JTable.AUTO_RESIZE_ALL_COLUMNS });
		resizeModeCombo.setSelectedItem(table.getAutoResizeMode());
		resizeModeCombo.setRenderer(new SubstanceDefaultListCellRenderer() {
			@Override
			public Component getListCellRendererComponent(JList list,
					Object value, int index, boolean isSelected,
					boolean cellHasFocus) {
				JLabel result = (JLabel) super.getListCellRendererComponent(
						list, value, index, isSelected, cellHasFocus);

				int iv = (Integer) value;
				switch (iv) {
				case JTable.AUTO_RESIZE_OFF:
					result.setText("off");
					break;
				case JTable.AUTO_RESIZE_NEXT_COLUMN:
					result.setText("next");
					break;
				case JTable.AUTO_RESIZE_SUBSEQUENT_COLUMNS:
					result.setText("subsequent");
					break;
				case JTable.AUTO_RESIZE_LAST_COLUMN:
					result.setText("last");
					break;
				case JTable.AUTO_RESIZE_ALL_COLUMNS:
					result.setText("all");
					break;
				}

				return result;
			}
		});

		resizeModeCombo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				int selected = (Integer) resizeModeCombo.getSelectedItem();
				table.setAutoResizeMode(selected);
			}
		});

		builder.append("Resize mode", resizeModeCombo);

		final JCheckBox hasRollovers = new JCheckBox("Has rollover effect");
		hasRollovers.setSelected(FadeConfigurationManager.getInstance()
				.fadeAllowed(FadeKind.ROLLOVER, table));
		hasRollovers.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (hasRollovers.isSelected()) {
					FadeConfigurationManager.getInstance().allowFades(
							FadeKind.ROLLOVER, table);
				} else {
					FadeConfigurationManager.getInstance().disallowFades(
							FadeKind.ROLLOVER, table);
				}
			}
		});
		builder.append("Rollovers", hasRollovers);

		builder.appendSeparator("Header settings");
		final JCheckBox useCurrentTheme = new JCheckBox("Global theme");
		useCurrentTheme.setSelected(true);
		useCurrentTheme.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.getTableHeader().putClientProperty(
						SubstanceLookAndFeel.THEME_PROPERTY,
						(useCurrentTheme.isSelected() ? null : "Bottle Green"));
				table.getTableHeader().repaint();
			}
		});
		builder.append("Theme", useCurrentTheme);

		final JCheckBox isAlwaysActive = new JCheckBox("Is always active");
		isAlwaysActive.setSelected(false);
		isAlwaysActive.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				table.getTableHeader().putClientProperty(
						SubstanceLookAndFeel.PAINT_ACTIVE_PROPERTY,
						(isAlwaysActive.isSelected() ? Boolean.TRUE : null));
				table.getTableHeader().repaint();
			}
		});
		builder.append("Active", isAlwaysActive);

		this.controlPanel = builder.getPanel();
	}
}