/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.check;

import java.awt.*;
import java.text.DecimalFormat;

import javax.swing.*;
import javax.swing.border.TitledBorder;

import org.jvnet.lafwidget.LafWidget;
import org.jvnet.lafwidget.text.PasswordStrengthChecker;
import org.jvnet.lafwidget.utils.LafConstants.PasswordStrength;
import org.jvnet.substance.SubstanceLookAndFeel;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import test.TwoColumnPanel;

public class TextFieldsPanel extends JPanel {
	private JPanel getContents() {
		FormLayout lm = new FormLayout(
				"right:pref, 4dlu, fill:default:grow(1), 4dlu,"
						+ "fill:default:grow(1), 4dlu, fill:default:grow(1)",
				"");
		// lm.setColumnGroups(new int[][] { { 3, 5, 7 } });
		DefaultFormBuilder builder = new DefaultFormBuilder(lm);
		builder.setDefaultDialogBorder();

		JLabel textLabel = new JLabel("Text fields");
		JLabel formattedTextLabel = new JLabel("Formatted text fields");
		JLabel passwordLabel = new JLabel("Password fields");
		textLabel.setFont(textLabel.getFont().deriveFont(Font.BOLD));
		formattedTextLabel.setFont(formattedTextLabel.getFont().deriveFont(
				Font.BOLD));
		passwordLabel.setFont(passwordLabel.getFont().deriveFont(Font.BOLD));

		builder.append("", textLabel);
		builder.append(formattedTextLabel, passwordLabel);

		JTextField jtf1 = new JTextField("sample text");
		jtf1.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.FALSE);
		builder.append("Enabled", jtf1);

		JFormattedTextField jftf1 = new JFormattedTextField(new DecimalFormat(
				"#,##0.0000"));
		jftf1.setText("2,430.0000");
		jftf1.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.FALSE);
		JPasswordField jpf1 = new JPasswordField("password", 10);
		jpf1.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.FALSE);
		builder.append(jftf1, jpf1);

		JTextField jtf2 = new JTextField("sample text");
		jtf2.setEditable(false);
		builder.append("Not editable", jtf2);

		JFormattedTextField jftf2 = new JFormattedTextField(new DecimalFormat(
				"#,##0.0000"));
		jftf2.setText("2,430.0000");
		jftf2.setEditable(false);
		jftf2.putClientProperty(SubstanceLookAndFeel.NO_EXTRA_ELEMENTS,
				Boolean.TRUE);
		jftf2.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.TRUE);
		JPasswordField jpf2 = new JPasswordField("password", 10);
		jpf2.setEditable(false);
		builder.append(jftf2, jpf2);

		JTextField jtf3 = new JTextField("sample text");
		jtf3.setEnabled(false);
		builder.append("Disabled", jtf3);

		JFormattedTextField jftf3 = new JFormattedTextField(new DecimalFormat(
				"#,##0.0000"));
		jftf3.setText("2,430.0000");
		jftf3.setEnabled(false);
		jftf3.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.TRUE);
		JPasswordField jpf3 = new JPasswordField("password", 10);
		jpf3.setEnabled(false);
		builder.append(jftf3, jpf3);

		JTextField jtf4 = new JTextField("sample text");
		jtf4.putClientProperty(LafWidget.TEXT_SELECT_ON_FOCUS, Boolean.TRUE);
		builder.append("Select all on focus", jtf4);

		JFormattedTextField jftf4 = new JFormattedTextField(new DecimalFormat(
				"#,##0.0000"));
		jftf4.setText("2,430.0000");
		jftf4.putClientProperty(LafWidget.TEXT_SELECT_ON_FOCUS, Boolean.TRUE);
		builder.append(jftf4);
		builder.nextLine();

		JTextField jtf4_1 = new JTextField("sample text");
		jtf4_1.putClientProperty(LafWidget.TEXT_FLIP_SELECT_ON_ESCAPE,
				Boolean.TRUE);
		builder.append("Flip selection on ESC", jtf4_1);

		JFormattedTextField jftf4_1 = new JFormattedTextField(
				new DecimalFormat("#,##0.0000"));
		jftf4_1.setText("2,430.0000");
		jftf4_1.putClientProperty(LafWidget.TEXT_FLIP_SELECT_ON_ESCAPE,
				Boolean.TRUE);
		builder.append(jftf4_1);
		builder.nextLine();

		JTextField jtf5 = new JTextField("sample text");
		jtf5.putClientProperty(LafWidget.TEXT_EDIT_CONTEXT_MENU, Boolean.TRUE);
		builder.append("With context menu", jtf5);

		JFormattedTextField jftf5 = new JFormattedTextField(new DecimalFormat(
				"#,##0.0000"));
		jftf5.setText("2,430.0000");
		jftf5.putClientProperty(LafWidget.TEXT_EDIT_CONTEXT_MENU, Boolean.TRUE);
		builder.append(jftf5);
		builder.nextLine();

		JTextField jtf6 = new JTextField("sample text");
		jtf6.setMargin(new Insets(2, 2, 2, 2));
		builder.append("With margin", jtf6);

		JFormattedTextField jftf6 = new JFormattedTextField(new DecimalFormat(
				"#,##0.0000"));
		jftf6.setText("2,430.0000");
		jftf6.setMargin(new Insets(2, 2, 2, 2));
		jftf6.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.FALSE);
		JPasswordField jpf6 = new JPasswordField("password", 10);
		jpf6.setMargin(new Insets(2, 2, 2, 2));
		builder.append(jftf6, jpf6);

		JPasswordField jpfEmptyEchoChar = new JPasswordField("password", 10);
		jpfEmptyEchoChar.setEchoChar((char) 0);
		builder.append("Empty echo char", new JLabel());
		builder.append(new JLabel(), jpfEmptyEchoChar);

		JPasswordField jpfStrengthChecker = new JPasswordField("password", 10);
		try {
			jpfStrengthChecker.putClientProperty(
					LafWidget.PASSWORD_STRENGTH_CHECKER,
					new PasswordStrengthChecker() {
						public PasswordStrength getStrength(char[] password) {
							if (password == null)
								return PasswordStrength.WEAK;
							int length = password.length;
							if (length < 3)
								return PasswordStrength.WEAK;
							if (length < 6)
								return PasswordStrength.MEDIUM;
							return PasswordStrength.STRONG;
						}

						public String getDescription(PasswordStrength strength) {
							if (strength == PasswordStrength.WEAK)
								return "<html>This password is <b>way</b> too weak</html>";
							if (strength == PasswordStrength.MEDIUM)
								return "<html>Come on, you can do<br> a little better than that</html>";
							if (strength == PasswordStrength.STRONG)
								return "OK";
							return null;
						}
					});
			builder.append("Strength checker", new JLabel());
			builder.append(new JLabel(), jpfStrengthChecker);
		} catch (Throwable t) {
		}

		JPasswordField jpfTwoEchoChar = new JPasswordField("password", 10);
		jpfTwoEchoChar.putClientProperty(
				SubstanceLookAndFeel.PASSWORD_ECHO_PER_CHAR, new Integer(2));
		builder.append("Two echo chars", new JLabel());
		builder.append(new JLabel(), jpfTwoEchoChar);

		JLabel editorPaneLabel = new JLabel("Editor panes");
		JLabel textAreaLabel = new JLabel("Text areas");
		JLabel textPaneLabel = new JLabel("Text panes");
		editorPaneLabel
				.setFont(editorPaneLabel.getFont().deriveFont(Font.BOLD));
		textAreaLabel.setFont(textAreaLabel.getFont().deriveFont(Font.BOLD));
		textPaneLabel.setFont(textPaneLabel.getFont().deriveFont(Font.BOLD));

		builder.append("", editorPaneLabel);
		builder.append(textAreaLabel, textPaneLabel);

		JEditorPane jep1 = new JEditorPane("text/html;",
				"Sample <b>content</b><br> <u>text</u>");
		jep1.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.FALSE);
		builder.append("Enabled", jep1);

		JTextArea jta1 = new JTextArea("Sample content text", 3, 20);
		jta1.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.FALSE);
		JTextPane jtp1 = new JTextPane();
		jtp1.replaceSelection("Sample content text");
		jtp1.setPreferredSize(new Dimension(100, 40));
		jtp1.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.FALSE);
		builder.append(jta1, jtp1);

		JEditorPane jep2 = new JEditorPane("text/html;",
				"Sample <b>content</b><br> <u>text</u>");
		jep2.setEditable(false);
		builder.append("Not editable", jep2);

		JTextArea jta2 = new JTextArea("Sample content text", 3, 20);
		jta2.setEditable(false);
		JTextPane jtp2 = new JTextPane();
		jtp2.replaceSelection("Sample content text");
		jtp2.setPreferredSize(new Dimension(100, 40));
		jtp2.setEditable(false);
		builder.append(jta2, jtp2);

		JEditorPane jep3 = new JEditorPane("text/html;",
				"Sample <b>content</b><br> <u>text</u>");
		jep3.setEnabled(false);
		builder.append("Disabled", jep3);

		JTextArea jta3 = new JTextArea("Sample content text", 3, 20);
		jta3.setEnabled(false);
		JTextPane jtp3 = new JTextPane();
		jtp3.replaceSelection("Sample content text");
		jtp3.setPreferredSize(new Dimension(100, 40));
		jtp3.setEnabled(false);
		builder.append(jta3, jtp3);

		JEditorPane jep4 = new JEditorPane("text/html;",
				"Sample <b>content</b><br> <u>text</u>");
		jep4.setMargin(new Insets(2, 2, 2, 2));
		builder.append("With margin", jep4);

		JTextArea jta4 = new JTextArea("Sample content text", 3, 20);
		jta4.setMargin(new Insets(2, 2, 2, 2));
		JTextPane jtp4 = new JTextPane();
		jtp4.replaceSelection("Sample content text");
		jtp4.setPreferredSize(new Dimension(100, 40));
		jtp4.setMargin(new Insets(2, 2, 2, 2));
		builder.append(jta4, jtp4);

		return builder.getPanel();
	}

	public TextFieldsPanel() {
		this.putClientProperty(SubstanceLookAndFeel.WATERMARK_IGNORE,
				Boolean.TRUE);

		this.setLayout(new BorderLayout());
		this.add(new JScrollPane(getContents()), BorderLayout.CENTER);
	}
}