/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.samples.substance.api;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;

import javax.swing.*;

import org.jvnet.substance.SubstanceImageCreator;
import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.theme.SubstanceTheme;
import org.jvnet.substance.title.TitleButtonCallback;
import org.jvnet.substance.title.TitleButtonInfo;

/**
 * Test application that shows the use of the
 * {@link SubstanceLookAndFeel#removeRootPaneCustomTitleButtons(JRootPane)} and
 * {@link SubstanceLookAndFeel#setRootPaneCustomTitleButtons(JRootPane, java.util.List)}
 * API.
 * 
 * @author Kirill Grouchnikov
 * @see SubstanceLookAndFeel#removeRootPaneCustomTitleButtons(JRootPane)
 * @see SubstanceLookAndFeel#setRootPaneCustomTitleButtons(JRootPane,
 *      java.util.List)
 */
public class SetRemoveRootPaneCustomTitleButtons extends JFrame {
	/**
	 * Creates the main frame for <code>this</code> sample.
	 */
	public SetRemoveRootPaneCustomTitleButtons() {
		super("Root pane custom title buttons");

		this.setLayout(new BorderLayout());

		JPanel controls = new JPanel(new FlowLayout(FlowLayout.RIGHT));

		controls.add(new JLabel("Root pane custom title buttons"));
		final JButton set = new JButton("Set");
		final JButton remove = new JButton("Remove");
		remove.setEnabled(false);
		set.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// create info on two sample custom title buttons.
				TitleButtonInfo tbInfo1 = new TitleButtonInfo();
				tbInfo1.setActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						JOptionPane.showMessageDialog(
								SetRemoveRootPaneCustomTitleButtons.this,
								"Custom button 1 activated");
					}
				});
				tbInfo1.setTooltipText("Tooltip for custom button 1");
				tbInfo1.setButtonCallback(new TitleButtonCallback() {
					public Icon getTitleButtonIcon(
							SubstanceTheme currSubstanceTheme,
							int iconMaxWidth, int iconMaxHeight) {
						return new ImageIcon(SubstanceImageCreator.overlayEcho(
								SubstanceImageCreator
										.getArrow(9, 5, 2.0f, SwingConstants.WEST,
												currSubstanceTheme), 1, 1));
					}
				});

				TitleButtonInfo tbInfo2 = new TitleButtonInfo();
				tbInfo2.setActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						JOptionPane.showMessageDialog(
								SetRemoveRootPaneCustomTitleButtons.this,
								"Custom button 2 activated");
					}
				});
				tbInfo2.setTooltipText("Tooltip for custom button 2");
				tbInfo2.setButtonCallback(new TitleButtonCallback() {
					public Icon getTitleButtonIcon(
							SubstanceTheme currSubstanceTheme,
							int iconMaxWidth, int iconMaxHeight) {
						return new ImageIcon(SubstanceImageCreator.overlayEcho(
								SubstanceImageCreator
										.getArrow(9, 5, 2.0f, SwingConstants.EAST,
												currSubstanceTheme), 1, 1));
					}
				});

				java.util.List<TitleButtonInfo> tbInfoList = new LinkedList<TitleButtonInfo>();
				tbInfoList.add(tbInfo1);
				tbInfoList.add(tbInfo2);
				
				// set custom title buttons
				SubstanceLookAndFeel.setRootPaneCustomTitleButtons(
						getRootPane(), tbInfoList);

				set.setEnabled(false);
				remove.setEnabled(true);
			}
		});
		remove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// remove custom title buttons
				SubstanceLookAndFeel
						.removeRootPaneCustomTitleButtons(getRootPane());
				remove.setEnabled(false);
				set.setEnabled(true);
			}
		});

		controls.add(set);
		controls.add(remove);
		this.add(controls, BorderLayout.SOUTH);

		this.setSize(400, 200);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * The main method for <code>this</code> sample. The arguments are
	 * ignored.
	 * 
	 * @param args
	 *            Ignored.
	 * @throws Exception
	 *             If some exception occured. Note that there is no special
	 *             treatment of exception conditions in <code>this</code>
	 *             sample code.
	 */
	public static void main(String[] args) throws Exception {
		UIManager.setLookAndFeel(new SubstanceLookAndFeel());
		JFrame.setDefaultLookAndFeelDecorated(true);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new SetRemoveRootPaneCustomTitleButtons().setVisible(true);
			}
		});
	}
}
