/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.samples.substance.clientprop;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.button.ClassicButtonShaper;
import org.jvnet.substance.utils.SubstanceConstants.TabAnimationKind;

import test.Check;

/**
 * Test application that shows the use of the
 * {@link SubstanceLookAndFeel#TABBED_PANE_TAB_ANIMATION_KIND} client property.
 * 
 * @author Kirill Grouchnikov
 * @see SubstanceLookAndFeel#TABBED_PANE_TAB_ANIMATION_KIND
 */
public class TabbedPaneTabAnimation extends JFrame {
	/**
	 * Creates the main frame for <code>this</code> sample.
	 */
	public TabbedPaneTabAnimation() {
		super("Tabbed pane tab animation");

		this.setLayout(new BorderLayout());

		// create tabbed pane with a few tabs
		final JTabbedPane jtp = new JTabbedPane();
		jtp.addTab("First", Check.getIcon("flag_mexico"), new JPanel());
		jtp.addTab("Second", Check.getIcon("flag_sweden"), new JPanel());

		this.add(jtp, BorderLayout.CENTER);

		JPanel controls = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		final JComboBox animationKindCombo = new JComboBox(new Object[] {
				"stop", "load", "error", "warning" });
		JButton setAnimationKindButton = new JButton("Set");
		setAnimationKindButton.putClientProperty(
				SubstanceLookAndFeel.BUTTON_SHAPER_PROPERTY,
				new ClassicButtonShaper());
		setAnimationKindButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				Component comp = jtp.getComponentAt(1);
				String selectedAnimationKind = (String) animationKindCombo
						.getSelectedItem();
				TabAnimationKind animKind = null;

				// compute the new animation kind to set on the second tab
				// component
				if ("load".equals(selectedAnimationKind))
					animKind = TabAnimationKind.LOADING;
				if ("error".equals(selectedAnimationKind))
					animKind = TabAnimationKind.ERROR;
				if ("warning".equals(selectedAnimationKind))
					animKind = TabAnimationKind.WARNING;
				if ((comp != null) && (comp instanceof JComponent)) {
					JComponent jc = (JComponent) comp;
					// set the selected animation kind
					jc
							.putClientProperty(
									SubstanceLookAndFeel.TABBED_PANE_TAB_ANIMATION_KIND,
									animKind);
				}

			}
		});

		controls.add(animationKindCombo);
		controls.add(setAnimationKindButton);
		this.add(controls, BorderLayout.SOUTH);

		this.setSize(400, 200);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * The main method for <code>this</code> sample. The arguments are
	 * ignored.
	 * 
	 * @param args
	 *            Ignored.
	 * @throws Exception
	 *             If some exception occured. Note that there is no special
	 *             treatment of exception conditions in <code>this</code>
	 *             sample code.
	 */
	public static void main(String[] args) throws Exception {
		UIManager.setLookAndFeel(new SubstanceLookAndFeel());
		JFrame.setDefaultLookAndFeelDecorated(true);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new TabbedPaneTabAnimation().setVisible(true);
			}
		});
	}
}
