/*
 * Copyright (c) 2005-2007 Substance Kirill Grouchnikov. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  o Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer. 
 *     
 *  o Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution. 
 *     
 *  o Neither the name of Substance Kirill Grouchnikov nor the names of 
 *    its contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission. 
 *     
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */
package test.samples.substance.clientprop;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.*;

import org.jvnet.substance.SubstanceLookAndFeel;
import org.jvnet.substance.utils.SubstanceCoreUtilities;

import test.Check;

/**
 * Test application that shows the use of the
 * {@link SubstanceLookAndFeel#TOOLBAR_BUTTON_FLAT} client property.
 * 
 * @author Kirill Grouchnikov
 * @see SubstanceLookAndFeel#TOOLBAR_BUTTON_FLAT
 */
public class ToolbarButtonFlat extends JFrame {
	/**
	 * Creates the main frame for <code>this</code> sample.
	 */
	public ToolbarButtonFlat() {
		super("Toolbar button flat");

		this.setLayout(new BorderLayout());

		// create toolbar with a few toggle buttons and regular buttons
		final JToolBar toolbar = new JToolBar();
		JToggleButton tb1 = new JToggleButton(Check.getIcon("flag_mexico"));
		JToggleButton tb2 = new JToggleButton(Check.getIcon("flag_sweden"));
		JToggleButton tb3 = new JToggleButton(Check.getIcon("flag_hong_kong"));
		ButtonGroup bg = new ButtonGroup();
		bg.add(tb1);
		bg.add(tb2);
		bg.add(tb3);
		toolbar.add(tb1);
		toolbar.add(tb2);
		toolbar.add(tb3);

		toolbar.add(new JButton("test1", Check.getIcon("flag_mexico")));
		toolbar.add(new JButton("test2", Check.getIcon("flag_sweden")));
		toolbar.add(new JButton("test3", Check.getIcon("flag_hong_kong")));

		this.add(toolbar, BorderLayout.NORTH);

		JPanel controls = new JPanel(new FlowLayout(FlowLayout.RIGHT));
		final JCheckBox isToolbarButtonFlat = new JCheckBox(
				"toolbar buttons flat");
		isToolbarButtonFlat.setSelected(SubstanceCoreUtilities
				.isToolbarButtonFlat(tb1));
		isToolbarButtonFlat.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						// based on checkbox selection, mark toolbar to have
						// non-flat buttons
						toolbar.putClientProperty(
								SubstanceLookAndFeel.TOOLBAR_BUTTON_FLAT,
								isToolbarButtonFlat.isSelected() ? null
										: Boolean.FALSE);
						repaint();
					}
				});
			}
		});
		controls.add(isToolbarButtonFlat);
		this.add(controls, BorderLayout.SOUTH);

		this.setSize(400, 200);
		this.setLocationRelativeTo(null);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * The main method for <code>this</code> sample. The arguments are
	 * ignored.
	 * 
	 * @param args
	 *            Ignored.
	 * @throws Exception
	 *             If some exception occured. Note that there is no special
	 *             treatment of exception conditions in <code>this</code>
	 *             sample code.
	 */
	public static void main(String[] args) throws Exception {
		UIManager.setLookAndFeel(new SubstanceLookAndFeel());
		JFrame.setDefaultLookAndFeelDecorated(true);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new ToolbarButtonFlat().setVisible(true);
			}
		});
	}
}
